from pandas._libs.properties import cache_readonly as cache_readonly
from typing import Any, Callable, List, Mapping, Optional, Tuple, Type, TypeVar, Union

FuncType = Callable[..., Any]
F = TypeVar('F', bound=FuncType)

def deprecate(name: str, alternative: Callable[..., Any], version: str, alt_name: Optional[str]=..., klass: Optional[Type[Warning]]=..., stacklevel: int=..., msg: Optional[str]=...) -> Callable[..., Any]: ...
def deprecate_kwarg(old_arg_name: str, new_arg_name: Optional[str], mapping: Optional[Union[Mapping[Any, Any], Callable[[Any], Any]]]=..., stacklevel: int=...) -> Callable[..., Any]: ...
def rewrite_axis_style_signature(name: str, extra_params: List[Tuple[str, Any]]) -> Callable[..., Any]: ...

class Substitution:
    params: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __call__(self, func: F) -> F: ...
    def update(self, *args: Any, **kwargs: Any) -> None: ...

class Appender:
    addendum: Optional[str]
    join: Any = ...
    def __init__(self, addendum: Optional[str], join: str=..., indents: int=...) -> None: ...
    def __call__(self, func: F) -> F: ...

def indent(text: Optional[str], indents: int=...) -> str: ...



from pandas import compat as compat
from pandas.core import ops as ops
from pandas.core.arrays import PandasArray as PandasArray
from pandas.core.construction import extract_array as extract_array
from pandas.core.dtypes.base import ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.common import pandas_dtype as pandas_dtype
from pandas.core.dtypes.dtypes import register_extension_dtype as register_extension_dtype
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.core.dtypes.inference import is_array_like as is_array_like

from pandas.core.missing import isna as isna
from typing import Any, Optional, Type

class StringDtype(ExtensionDtype):
    name: str = ...
    na_value: Any = ...
    @property
    def type(self) -> Type[Any]: ...
    @classmethod
    def construct_array_type(cls: Any) -> Type[StringArray]: ...
    def __from_arrow__(self, array: Any) -> Any: ...

class StringArray(PandasArray):
    def __init__(self, values: Any, copy: bool = ...) -> None: ...
    def __arrow_array__(self, type: Optional[Any] = ...) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def fillna(self, value: Optional[Any] = ..., method: Optional[Any] = ..., limit: Optional[Any] = ...) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def value_counts(self, dropna: bool = ...) -> Any: ...



from dateutil.relativedelta import FR as FR, SA as SA, SU as SU, TU as TU, WE as WE
from pandas import DateOffset as DateOffset, Series as Series, Timestamp as Timestamp, date_range as date_range
from pandas.errors import PerformanceWarning as PerformanceWarning
from pandas.tseries.offsets import Day as Day, Easter as Easter
from typing import Any, List, Optional

def next_monday(dt: Any) -> Any: ...
def next_monday_or_tuesday(dt: Any) -> Any: ...
def previous_friday(dt: Any) -> Any: ...
def sunday_to_monday(dt: Any) -> Any: ...
def weekend_to_monday(dt: Any) -> Any: ...
def nearest_workday(dt: Any) -> Any: ...
def next_workday(dt: Any) -> Any: ...
def previous_workday(dt: Any) -> Any: ...
def before_nearest_workday(dt: Any) -> Any: ...
def after_nearest_workday(dt: Any) -> Any: ...

class Holiday:
    name: Any = ...
    year: Any = ...
    month: Any = ...
    day: Any = ...
    offset: Any = ...
    start_date: Any = ...
    end_date: Any = ...
    observance: Any = ...
    days_of_week: Any = ...
    def __init__(self, name: Any, year: Optional[Any] = ..., month: Optional[Any] = ..., day: Optional[Any] = ..., offset: Optional[Any] = ..., observance: Optional[Any] = ..., start_date: Optional[Any] = ..., end_date: Optional[Any] = ..., days_of_week: Optional[Any] = ...) -> None: ...
    def dates(self, start_date: Any, end_date: Any, return_name: bool = ...) -> Any: ...

holiday_calendars: Any

def register(cls: Any) -> None: ...
def get_calendar(name: Any) -> Any: ...

class HolidayCalendarMetaClass(type):
    def __new__(cls, clsname: Any, bases: Any, attrs: Any) -> Any: ...

class AbstractHolidayCalendar(metaclass=HolidayCalendarMetaClass):
    rules: List[Holiday] = ...
    start_date: Any = ...
    end_date: Any = ...
    name: Any = ...
    def __init__(self, name: Optional[Any] = ..., rules: Optional[Any] = ...) -> None: ...
    def rule_from_name(self, name: Any) -> Any: ...
    def holidays(self, start: Optional[Any] = ..., end: Optional[Any] = ..., return_name: bool = ...) -> Any: ...
    @staticmethod
    def merge_class(base: Any, other: Any) -> Any: ...
    def merge(self, other: Any, inplace: bool = ...) -> Any: ...

USMemorialDay: Any
USLaborDay: Any
USColumbusDay: Any
USThanksgivingDay: Any
USMartinLutherKingJr: Any
USPresidentsDay: Any
GoodFriday: Any
EasterMonday: Any

class USFederalHolidayCalendar(AbstractHolidayCalendar):
    rules: Any = ...

def HolidayCalendarFactory(name: Any, base: Any, other: Any, base_class: Any = ...) -> Any: ...



import numpy as np
from pandas._config import get_option as get_option
from pandas._typing import ArrayLike as ArrayLike, Dtype as Dtype, Ordered as Ordered, Scalar as Scalar
from pandas.core import ops as ops
from pandas.core.accessor import PandasDelegate as PandasDelegate, delegate_names as delegate_names
from pandas.core.algorithms import factorize as factorize, take as take, take_1d as take_1d, unique1d as unique1d
from pandas.core.arrays.base import ExtensionArray as ExtensionArray
from pandas.core.base import NoNewAttributesMixin as NoNewAttributesMixin, PandasObject as PandasObject
from pandas.core.construction import array as array, extract_array as extract_array, sanitize_array as sanitize_array
from pandas.core.dtypes.cast import coerce_indexer_dtype as coerce_indexer_dtype, maybe_infer_to_datetimelike as maybe_infer_to_datetimelike
from pandas.core.dtypes.common import ensure_int64 as ensure_int64, ensure_object as ensure_object, ensure_platform_int as ensure_platform_int, is_categorical_dtype as is_categorical_dtype, is_datetime64_dtype as is_datetime64_dtype, is_dict_like as is_dict_like, is_dtype_equal as is_dtype_equal, is_extension_array_dtype as is_extension_array_dtype, is_integer_dtype as is_integer_dtype, is_iterator as is_iterator, is_list_like as is_list_like, is_object_dtype as is_object_dtype, is_scalar as is_scalar, is_sequence as is_sequence, is_timedelta64_dtype as is_timedelta64_dtype, needs_i8_conversion as needs_i8_conversion
from pandas.core.dtypes.dtypes import CategoricalDtype as CategoricalDtype
from pandas.core.dtypes.generic import ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.core.dtypes.inference import is_hashable as is_hashable
from pandas.core.dtypes.missing import isna as isna, notna as notna
from pandas.core.ops.common import unpack_zerodim_and_defer as unpack_zerodim_and_defer
from pandas.core.sorting import nargsort as nargsort
from pandas.io.formats import console as console
from pandas.util._decorators import Appender as Appender, Substitution as Substitution, cache_readonly as cache_readonly, deprecate_kwarg as deprecate_kwarg
from pandas.util._validators import validate_bool_kwarg as validate_bool_kwarg, validate_fillna_kwargs as validate_fillna_kwargs
from typing import Any, List, Optional

def contains(cat: Any, key: Any, container: Any) -> Any: ...

class Categorical(ExtensionArray, PandasObject):
    __array_priority__: int = ...
    def __init__(self, values: Any, categories: Optional[Any] = ..., ordered: Optional[Any] = ..., dtype: Optional[Any] = ..., fastpath: bool = ...) -> None: ...
    @property
    def categories(self) -> Any: ...
    @categories.setter
    def categories(self, categories: Any) -> None: ...
    @property
    def ordered(self) -> Ordered: ...
    @property
    def dtype(self) -> CategoricalDtype: ...
    def copy(self) -> Categorical: ...
    def astype(self, dtype: Dtype, copy: bool=...) -> ArrayLike: ...
    def size(self) -> int: ...
    def itemsize(self) -> int: ...
    def tolist(self) -> List[Scalar]: ...
    to_list: Any = ...
    @classmethod
    def from_codes(cls, codes: Any, categories: Optional[Any] = ..., ordered: Optional[Any] = ..., dtype: Optional[Any] = ...) -> Any: ...
    codes: Any = ...
    def set_ordered(self, value: Any, inplace: bool = ...) -> Any: ...
    def as_ordered(self, inplace: bool = ...) -> Any: ...
    def as_unordered(self, inplace: bool = ...) -> Any: ...
    def set_categories(self, new_categories: Any, ordered: Optional[Any] = ..., rename: bool = ..., inplace: bool = ...) -> Any: ...
    def rename_categories(self, new_categories: Any, inplace: bool = ...) -> Any: ...
    def reorder_categories(self, new_categories: Any, ordered: Optional[Any] = ..., inplace: bool = ...) -> Any: ...
    def add_categories(self, new_categories: Any, inplace: bool = ...) -> Any: ...
    def remove_categories(self, removals: Any, inplace: bool = ...) -> Any: ...
    def remove_unused_categories(self, inplace: bool = ...) -> Any: ...
    def map(self, mapper: Any) -> Any: ...
    __eq__: Any = ...
    __ne__: Any = ...
    __lt__: Any = ...
    __gt__: Any = ...
    __le__: Any = ...
    __ge__: Any = ...
    @property
    def shape(self) -> Any: ...
    def shift(self, periods: Any, fill_value: Optional[Any] = ...) -> Any: ...  # type: ignore
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __array_ufunc__(self, ufunc: Any, method: Any, *inputs: Any, **kwargs: Any) -> Any: ...
    @property
    def T(self) -> Any: ...
    @property
    def nbytes(self) -> Any: ...
    def memory_usage(self, deep: bool = ...) -> Any: ...
    def searchsorted(self, value: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
    def isna(self) -> Any: ...
    isnull: Any = ...
    def notna(self) -> Any: ...
    notnull: Any = ...
    def put(self, *args: Any, **kwargs: Any) -> None: ...
    def dropna(self) -> Any: ...
    def value_counts(self, dropna: bool = ...) -> Any: ...
    def check_for_ordered(self, op: Any) -> None: ...
    def argsort(self, ascending: bool = ..., kind: str = ..., *args: Any, **kwargs: Any) -> Any: ...
    def sort_values(self, inplace: bool = ..., ascending: bool = ..., na_position: str = ...) -> Any: ...
    def view(self, dtype: Optional[Any] = ...) -> Any: ...
    def to_dense(self) -> Any: ...
    def fillna(self, value: Optional[Any] = ..., method: Optional[Any] = ..., limit: Optional[Any] = ...) -> Any: ...
    def take(self, indexer: Any, allow_fill: bool=..., fill_value: Any = ...) -> Any: ...
    def take_nd(self, indexer: Any, allow_fill: bool=..., fill_value: Any = ...) -> Any: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Any: ...
    def __contains__(self, key: Any) -> bool: ...
    def __getitem__(self, key: Any) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...
    def min(self, skipna: bool = ...) -> Any: ...
    def max(self, skipna: bool = ...) -> Any: ...
    def mode(self, dropna: bool = ...) -> Any: ...
    def unique(self) -> Any: ...
    def equals(self, other: Any) -> Any: ...
    def is_dtype_equal(self, other: Any) -> Any: ...
    def describe(self) -> Any: ...
    def repeat(self, repeats: Any, axis: Optional[Any] = ...) -> Any: ...
    def isin(self, values: Any) -> Any: ...
    def replace(self, to_replace: Any, value: Any, inplace: bool=...) -> Any: ...

class CategoricalAccessor(PandasDelegate, PandasObject, NoNewAttributesMixin):
    def __init__(self, data: Any) -> None: ...
    @property
    def codes(self) -> Any: ...

def factorize_from_iterable(values: Any) -> Any: ...
def factorize_from_iterables(iterables: Any) -> Any: ...



from pandas._typing import Scalar as Scalar
from typing import Any, Callable, Dict, Optional, Tuple

def make_rolling_apply(func: Callable[..., Scalar], args: Tuple[Any, ...], nogil: bool, parallel: bool, nopython: bool) -> Any: ...
def generate_numba_apply_func(args: Tuple[Any, ...], kwargs: Dict[str, Any], func: Callable[..., Scalar], engine_kwargs: Optional[Dict[str, bool]]) -> Any: ...



from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.errors import AbstractMethodError as AbstractMethodError
from typing import Any, List, Optional, Type

class ExtensionDtype:
    def __eq__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    def __ne__(self, other: Any) -> bool: ...
    @property
    def na_value(self) -> Any: ...
    @property
    def type(self) -> Type[Any]: ...
    @property
    def kind(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def names(self) -> Optional[List[str]]: ...
    @classmethod
    def construct_array_type(cls) -> Any: ...
    @classmethod
    def construct_from_string(cls: Any, string: str) -> Any: ...
    @classmethod
    def is_dtype(cls: Any, dtype: Any) -> bool: ...



from typing import Any

class TablePlotter:
    cell_width: Any = ...
    cell_height: Any = ...
    font_size: Any = ...
    def __init__(self, cell_width: float=..., cell_height: float=..., font_size: float=...) -> None: ...
    def plot(self, left: Any, right: Any, labels: Any = ..., vertical: bool=...) -> Any: ...



import sys
from typing import Any, Optional, Union, overload

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

from dateutil.tz import tzfile
from datetime import tzinfo

from pandas._libs.tslibs.period import Period
from pandas._typing import TimestampConvertible

OptInt = Optional[int]

Fold = Literal[0, 1]

class Timestamp:
    @overload
    def __init__(self, ts_input: TimestampConvertible, freq: Optional[str] = ..., tz: Optional[Union[str, tzinfo, tzfile]] = ..., unit: Optional[str] = ..., tzinfo: Optional[tzinfo] = ..., fold: Optional[Fold] = ...): ...
    @overload
    def __init__(self, year: OptInt = ..., month: OptInt = ..., day: OptInt = ..., hour: OptInt = ..., minute: OptInt = ..., second: OptInt = ..., microsecond: OptInt = ..., nanosecond: OptInt = ..., tz: Optional[Union[str, tzinfo, tzfile]] = ..., tzinfo: Optional[tzinfo] = ..., fold: Optional[Fold] = ..., freq: Optional[str] = ...): ...
    def to_period(self, freq: Optional[str]) -> Period: ...
    def to_julian_date(self) -> float: ...
    def tz_localize(self, tz: Any = ..., ambigious: Any = ..., nonexistent: Any = ...) -> Timestamp: ...
    def tz_convert(self, tz: Any) -> Timestamp: ...
    def astimezone(self, tz: Any) -> Timestamp: ...
    def replace(self, year: OptInt = ..., month: OptInt = ..., day: OptInt = ..., hour: OptInt = ..., minute: OptInt = ..., second: OptInt = ..., microsecond: OptInt = ..., nanosecond: OptInt = ..., tzinfo: Optional[tzinfo] = ..., fold: Fold = ...) -> Timestamp: ...
    def round(self, freq: str, ambiguous: Any = ..., nonexistent: Any = ...) -> Timestamp: ...
    def ceil(self, freq: str, ambiguous: Any = ...,nonexistent: Any = ...) -> Timestamp: ...
    def floor(self, freq: str, ambiguous: Any = ...,nonexistent: Any = ...) -> Timestamp: ...
    def isoformat(self, sep: str = ...) -> str: ...
    def day_name(self, locale: Optional[str]) -> str: ...
    def month_name(self, locale: Optional[str]) -> str: ...
    def normalize(self) -> Timestamp: ...

    @classmethod
    def utcnow(cls) -> Timestamp: ...
    @classmethod
    def utcfromtimestamp(cls, ts: Timestamp) -> Timestamp: ...
    @classmethod
    def fromtimestamp(cls, ts: Timestamp) -> Timestamp: ...
    @classmethod
    def combine(cls, date: Any, time: Any) -> Timestamp: ...
    @classmethod
    def now(cls, tz: Optional[Any] = ...) -> Timestamp: ...
    @classmethod
    def today(cls, tz: Optional[Any] = ...) -> Timestamp: ...
    @classmethod
    def fromordinal(cls, ordinal: Any, freq: Optional[Any] = ..., tz: Optional[Any] = ...) -> Timestamp: ...
    @property
    def dayofweek(self) -> int: ...
    @property
    def dayofyear(self) -> int: ...
    @property
    def quarter(self) -> int: ...
    @property
    def days_in_month(self) -> int: ...
    @property
    def daysinmonth(self) -> int: ...
    @property
    def week(self) -> int: ...
    @property
    def weekofyear(self) -> int: ...
    @property
    def freqstr(self) -> str: ...
    @property
    def is_month_end(self) -> bool: ...
    @property
    def is_month_start(self) -> bool: ...
    @property
    def is_quarter_start(self) -> bool: ...
    @property
    def is_quarter_end(self) -> bool: ...
    @property
    def is_year_start(self) -> bool: ...
    @property
    def is_year_end(self) -> bool: ...
    @property
    def is_leap_year(self) -> bool: ...
    @property
    def tz(self) -> tzinfo: ...


def integer_op_not_supported(obj: Any) -> Any: ...



import pyarrow
from typing import Any

def pyarrow_array_to_numpy_and_mask(arr: Any, dtype: Any) -> Any: ...

class ArrowPeriodType(pyarrow.ExtensionType):   # type: ignore
    def __init__(self, freq: Any) -> None: ...
    @property
    def freq(self) -> Any: ...
    def __arrow_ext_serialize__(self) -> Any: ...
    @classmethod
    def __arrow_ext_deserialize__(cls, storage_type: Any, serialized: Any) -> Any: ...
    def __eq__(self, other: Any) -> Any: ...
    def __hash__(self) -> Any: ...

class ArrowIntervalType(pyarrow.ExtensionType):   # type: ignore
    def __init__(self, subtype: Any, closed: Any) -> None: ...
    @property
    def subtype(self) -> Any: ...
    @property
    def closed(self) -> Any: ...
    def __arrow_ext_serialize__(self) -> Any: ...
    @classmethod
    def __arrow_ext_deserialize__(cls, storage_type: Any, serialized: Any) -> Any: ...
    def __eq__(self, other: Any) -> Any: ...
    def __hash__(self) -> Any: ...



from pandas import option_context as option_context
from pandas.core.dtypes.generic import ABCMultiIndex as ABCMultiIndex
from pandas.io.common import is_url as is_url
from pandas.io.formats.format import DataFrameFormatter as DataFrameFormatter, TableFormatter as TableFormatter, buffer_put_lines as buffer_put_lines, get_level_lengths as get_level_lengths
from pandas.io.formats.printing import pprint_thing as pprint_thing
from typing import Any, Dict, IO, Iterable, List, Optional, Tuple, Union

class HTMLFormatter(TableFormatter):
    indent_delta: int = ...
    fmt: Any = ...
    classes: Any = ...
    frame: Any = ...
    columns: Any = ...
    elements: Any = ...
    bold_rows: Any = ...
    escape: Any = ...
    show_dimensions: Any = ...
    border: Any = ...
    table_id: Any = ...
    render_links: Any = ...
    def __init__(self, formatter: DataFrameFormatter, classes: Optional[Union[str, List[str], Tuple[str, ...]]]=..., border: Optional[int]=...) -> None: ...
    @property
    def show_row_idx_names(self) -> bool: ...
    @property
    def show_col_idx_names(self) -> bool: ...
    @property
    def row_levels(self) -> int: ...
    @property
    def is_truncated(self) -> bool: ... # type: ignore
    @property
    def ncols(self) -> int: ...
    def write(self, s: Any, indent: int=...) -> None: ...
    def write_th(self, s: Any, header: bool=..., indent: int=..., tags: Optional[str]=...) -> None: ...
    def write_td(self, s: Any, indent: int=..., tags: Optional[str]=...) -> None: ...
    def write_tr(self, line: Iterable[Any], indent: int=..., indent_delta: int=..., header: bool=..., align: Optional[str]=..., tags: Optional[Dict[int, str]]=..., nindex_levels: int=...) -> None: ...
    def render(self) -> List[str]: ...
    def write_result(self, buf: IO[str]) -> None: ...

class NotebookFormatter(HTMLFormatter):
    def write_style(self) -> None: ...
    def render(self) -> List[str]: ...



from pandas.core.arrays.sparse.accessor import SparseAccessor as SparseAccessor, SparseFrameAccessor as SparseFrameAccessor
from pandas.core.arrays.sparse.array import BlockIndex as BlockIndex, IntIndex as IntIndex, SparseArray as SparseArray
from pandas.core.arrays.sparse.dtype import SparseDtype as SparseDtype



from pandas.core.dtypes.missing import notna as notna
from pandas.io.formats.printing import pprint_thing as pprint_thing
from typing import Any, Optional

def scatter_matrix(frame: Any, alpha: float = ..., figsize: Optional[Any] = ..., ax: Optional[Any] = ..., grid: bool = ..., diagonal: str = ..., marker: str = ..., density_kwds: Optional[Any] = ..., hist_kwds: Optional[Any] = ..., range_padding: float = ..., **kwds: Any) -> Any: ...
def radviz(frame: Any, class_column: Any, ax: Optional[Any] = ..., color: Optional[Any] = ..., colormap: Optional[Any] = ..., **kwds: Any) -> Any: ...
def andrews_curves(frame: Any, class_column: Any, ax: Optional[Any] = ..., samples: int = ..., color: Optional[Any] = ..., colormap: Optional[Any] = ..., **kwds: Any) -> Any: ...
def bootstrap_plot(series: Any, fig: Optional[Any] = ..., size: int = ..., samples: int = ..., **kwds: Any) -> Any: ...
def parallel_coordinates(frame: Any, class_column: Any, cols: Optional[Any] = ..., ax: Optional[Any] = ..., color: Optional[Any] = ..., use_columns: bool = ..., xticks: Optional[Any] = ..., colormap: Optional[Any] = ..., axvlines: bool = ..., axvlines_kwds: Optional[Any] = ..., sort_labels: bool = ..., **kwds: Any) -> Any: ...
def lag_plot(series: Any, lag: int = ..., ax: Optional[Any] = ..., **kwds: Any) -> Any: ...
def autocorrelation_plot(series: Any, ax: Optional[Any] = ..., **kwds: Any) -> Any: ...



from pandas.core.window.ewm import EWM as EWM
from pandas.core.window.expanding import Expanding as Expanding, ExpandingGroupby as ExpandingGroupby
from pandas.core.window.rolling import Rolling as Rolling, RollingGroupby as RollingGroupby, Window as Window



from pandas import compat as compat
from pandas.core.util.hashing import hash_array as hash_array, hash_pandas_object as hash_pandas_object
from pandas.util._decorators import Appender as Appender, Substitution as Substitution, cache_readonly as cache_readonly
from typing import Any

def __getattr__(name: Any) -> Any: ...

class _testing:
    def __getattr__(self, item: Any) -> Any: ...

testing: Any



import numpy as np

from typing import Any, Iterable, Union

def validate_args(fname: Any, args: Any, max_fname_arg_count: Any, compat_args: Any) -> None: ...
def validate_kwargs(fname: Any, kwargs: Any, compat_args: Any) -> None: ...
def validate_args_and_kwargs(fname: Any, args: Any, kwargs: Any, max_fname_arg_count: Any, compat_args: Any) -> None: ...
def validate_bool_kwarg(value: Any, arg_name: Any) -> Any: ...
def validate_axis_style_args(data: Any, args: Any, kwargs: Any, arg_name: Any, method_name: Any) -> Any: ...
def validate_fillna_kwargs(value: Any, method: Any, validate_scalar_dict_value: bool = ...) -> Any: ...
def validate_percentile(q: Union[float, Iterable[float]]) -> np.ndarray: ...



from collections import abc
from typing import Any, Optional

class _subheader_pointer: ...
class _column: ...

class SAS7BDATReader(abc.Iterator[Any]):
    index: Any = ...
    convert_dates: Any = ...
    blank_missing: Any = ...
    chunksize: Any = ...
    encoding: Any = ...
    convert_text: Any = ...
    convert_header_text: Any = ...
    default_encoding: str = ...
    compression: str = ...
    column_names_strings: Any = ...
    column_names: Any = ...
    column_formats: Any = ...
    columns: Any = ...
    handle: Any = ...
    def __init__(self, path_or_buf: Any, index: Optional[Any] = ..., convert_dates: bool = ..., blank_missing: bool = ..., chunksize: Optional[Any] = ..., encoding: Optional[Any] = ..., convert_text: bool = ..., convert_header_text: bool = ...) -> None: ...
    def column_data_lengths(self) -> Any: ...
    def column_data_offsets(self) -> Any: ...
    def column_types(self) -> Any: ...
    def close(self) -> None: ...
    def __next__(self) -> Any: ...
    def read(self, nrows: Optional[Any] = ...) -> Any: ...



from typing import Any

class PyperclipException(RuntimeError): ...

class PyperclipWindowsException(PyperclipException):
    def __init__(self, message: Any) -> None: ...

class CheckedCall:
    def __init__(self, f: Any) -> None: ...
    def __call__(self, *args: Any) -> Any: ...
    def __setattr__(self, key: Any, value: Any) -> None: ...

def determine_clipboard() -> Any: ...
def set_clipboard(clipboard: Any) -> None: ...

copy: Any
paste: Any
clipboard_get = paste
clipboard_set = copy



from collections import namedtuple

from pandas.plotting._matplotlib.core import LinePlot as LinePlot
from typing import Any, Optional

class BoxPlot(LinePlot):
    return_type: Any = ...
    def __init__(self, data: Any, return_type: str = ..., **kwargs: Any) -> None: ...
    def maybe_color_bp(self, bp: Any) -> None: ...
    @property
    def orientation(self) -> str: ...  # type: ignore
    @property
    def result(self) -> Any: ...

def boxplot(data: Any, column: Optional[Any] = ..., by: Optional[Any] = ..., ax: Optional[Any] = ..., fontsize: Optional[Any] = ..., rot: int = ..., grid: bool = ..., figsize: Optional[Any] = ..., layout: Optional[Any] = ..., return_type: Optional[Any] = ..., **kwds: Any) -> Any: ...
def boxplot_frame(self: Any, column: Optional[Any] = ..., by: Optional[Any] = ..., ax: Optional[Any] = ..., fontsize: Optional[Any] = ..., rot: int = ..., grid: bool = ..., figsize: Optional[Any] = ..., layout: Optional[Any] = ..., return_type: Optional[Any] = ..., **kwds: Any) -> Any: ...
def boxplot_frame_groupby(grouped: Any, subplots: bool = ..., column: Optional[Any] = ..., fontsize: Optional[Any] = ..., rot: int = ..., grid: bool = ..., ax: Optional[Any] = ..., figsize: Optional[Any] = ..., layout: Optional[Any] = ..., sharex: bool = ..., sharey: bool = ..., **kwds: Any) -> Any: ...



from pandas import DataFrame as DataFrame, Int64Index as Int64Index, RangeIndex as RangeIndex
from pandas.io.common import stringify_path as stringify_path
from typing import Any

def to_feather(df: DataFrame, path: Any) -> Any: ...
def read_feather(path: Any, columns: Any = ..., use_threads: bool=...) -> Any: ...



from pandas.core.api import DataFrame as DataFrame, Series as Series
from pandas.core.base import PandasObject as PandasObject

from pandas.core.dtypes.dtypes import DatetimeTZDtype as DatetimeTZDtype
from pandas.core.dtypes.missing import isna as isna
from pandas.core.tools.datetimes import to_datetime as to_datetime
from typing import Any, Optional

class SQLAlchemyRequired(ImportError): ...
class DatabaseError(IOError): ...

def execute(sql: Any, con: Any, cur: Optional[Any] = ..., params: Optional[Any] = ...) -> Any: ...
def read_sql_table(table_name: Any, con: Any, schema: Optional[Any] = ..., index_col: Optional[Any] = ..., coerce_float: bool = ..., parse_dates: Optional[Any] = ..., columns: Optional[Any] = ..., chunksize: Optional[Any] = ...) -> Any: ...
def read_sql_query(sql: Any, con: Any, index_col: Optional[Any] = ..., coerce_float: bool = ..., params: Optional[Any] = ..., parse_dates: Optional[Any] = ..., chunksize: Optional[Any] = ...) -> Any: ...
def read_sql(sql: Any, con: Any, index_col: Optional[Any] = ..., coerce_float: bool = ..., params: Optional[Any] = ..., parse_dates: Optional[Any] = ..., columns: Optional[Any] = ..., chunksize: Optional[Any] = ...) -> Any: ...
def to_sql(frame: Any, name: Any, con: Any, schema: Optional[Any] = ..., if_exists: str = ..., index: bool = ..., index_label: Optional[Any] = ..., chunksize: Optional[Any] = ..., dtype: Optional[Any] = ..., method: Optional[Any] = ...) -> None: ...
def has_table(table_name: Any, con: Any, schema: Optional[Any] = ...) -> Any: ...
table_exists = has_table

def pandasSQL_builder(con: Any, schema: Optional[Any] = ..., meta: Optional[Any] = ..., is_cursor: bool = ...) -> Any: ...

class SQLTable(PandasObject):
    name: Any = ...
    pd_sql: Any = ...
    prefix: Any = ...
    frame: Any = ...
    index: Any = ...
    schema: Any = ...
    if_exists: Any = ...
    keys: Any = ...
    dtype: Any = ...
    table: Any = ...
    def __init__(self, name: Any, pandas_sql_engine: Any, frame: Optional[Any] = ..., index: bool = ..., if_exists: str = ..., prefix: str = ..., index_label: Optional[Any] = ..., schema: Optional[Any] = ..., keys: Optional[Any] = ..., dtype: Optional[Any] = ...) -> None: ...
    def exists(self) -> Any: ...
    def sql_schema(self) -> Any: ...
    def create(self) -> None: ...
    def insert_data(self) -> Any: ...
    def insert(self, chunksize: Optional[Any] = ..., method: Optional[Any] = ...) -> None: ...
    def read(self, coerce_float: bool = ..., parse_dates: Optional[Any] = ..., columns: Optional[Any] = ..., chunksize: Optional[Any] = ...) -> Any: ...

class PandasSQL(PandasObject):
    def read_sql(self, *args: Any, **kwargs: Any) -> None: ...
    def to_sql(self, *args: Any, **kwargs: Any) -> None: ...

class SQLDatabase(PandasSQL):
    connectable: Any = ...
    meta: Any = ...
    def __init__(self, engine: Any, schema: Optional[Any] = ..., meta: Optional[Any] = ...) -> None: ...
    def run_transaction(self) -> None: ...
    def execute(self, *args: Any, **kwargs: Any) -> Any: ...
    def read_table(self, table_name: Any, index_col: Optional[Any] = ..., coerce_float: bool = ..., parse_dates: Optional[Any] = ..., columns: Optional[Any] = ..., schema: Optional[Any] = ..., chunksize: Optional[Any] = ...) -> Any: ...
    def read_query(self, sql: Any, index_col: Optional[Any] = ..., coerce_float: bool = ..., parse_dates: Optional[Any] = ..., params: Optional[Any] = ..., chunksize: Optional[Any] = ...) -> Any: ...
    read_sql: Any = ...
    def to_sql(self, frame: Any, name: Any, if_exists: str = ..., index: bool = ..., index_label: Optional[Any] = ..., schema: Optional[Any] = ..., chunksize: Optional[Any] = ..., dtype: Optional[Any] = ..., method: Optional[Any] = ...) -> None: ...   # type: ignore
    @property
    def tables(self) -> Any: ...
    def has_table(self, name: Any, schema: Optional[Any] = ...) -> Any: ...
    def get_table(self, table_name: Any, schema: Optional[Any] = ...) -> Any: ...
    def drop_table(self, table_name: Any, schema: Optional[Any] = ...) -> None: ...

class SQLiteTable(SQLTable):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def sql_schema(self) -> Any: ...
    def insert_statement(self) -> Any: ...

class SQLiteDatabase(PandasSQL):
    is_cursor: Any = ...
    con: Any = ...
    def __init__(self, con: Any, is_cursor: bool = ...) -> None: ...
    def run_transaction(self) -> None: ...
    def execute(self, *args: Any, **kwargs: Any) -> Any: ...
    def read_query(self, sql: Any, index_col: Optional[Any] = ..., coerce_float: bool = ..., params: Optional[Any] = ..., parse_dates: Optional[Any] = ..., chunksize: Optional[Any] = ...) -> Any: ...
    def to_sql(self, frame: Any, name: Any, if_exists: str = ..., index: bool = ..., index_label: Optional[Any] = ..., schema: Optional[Any] = ..., chunksize: Optional[Any] = ..., dtype: Optional[Any] = ..., method: Optional[Any] = ...) -> None: ...   # type: ignore
    def has_table(self, name: Any, schema: Optional[Any] = ...) -> Any: ...
    def get_table(self, table_name: Any, schema: Optional[Any] = ...) -> None: ...
    def drop_table(self, name: Any, schema: Optional[Any] = ...) -> None: ...

def get_schema(frame: Any, name: Any, keys: Optional[Any] = ..., con: Optional[Any] = ..., dtype: Optional[Any] = ...) -> Any: ...



from collections import abc
from pandas._typing import FilePathOrBuffer as FilePathOrBuffer
from pandas.core import algorithms as algorithms
from pandas.core.arrays import Categorical as Categorical
from pandas.core.dtypes.cast import astype_nansafe as astype_nansafe
from pandas.core.dtypes.dtypes import CategoricalDtype as CategoricalDtype
from pandas.core.dtypes.missing import isna as isna
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.indexes.api import Index as Index, MultiIndex as MultiIndex, RangeIndex as RangeIndex, ensure_index_from_sequences as ensure_index_from_sequences
from pandas.core.series import Series as Series
from pandas.errors import AbstractMethodError as AbstractMethodError, EmptyDataError as EmptyDataError, ParserError as ParserError, ParserWarning as ParserWarning
from pandas.io.date_converters import generic_parser as generic_parser
from pandas.util._decorators import Appender as Appender
from typing import Any, Optional, AnyStr

read_csv: Any
read_table: Any

def read_fwf(filepath_or_buffer: FilePathOrBuffer[AnyStr], colspecs: Any = ..., widths: Any = ..., infer_nrows: Any = ..., **kwds: Any) -> Any: ...

class TextFileReader(abc.Iterator[Any]):
    f: Any = ...
    orig_options: Any = ...
    engine: Any = ...
    chunksize: Any = ...
    nrows: Any = ...
    squeeze: Any = ...
    def __init__(self, f: Any, engine: Optional[Any] = ..., **kwds: Any) -> None: ...
    def close(self) -> None: ...
    def __next__(self) -> Any: ...
    def read(self, nrows: Optional[Any] = ...) -> Any: ...
    def get_chunk(self, size: Optional[Any] = ...) -> Any: ...

class ParserBase:
    names: Any = ...
    orig_names: Any = ...
    prefix: Any = ...
    index_col: Any = ...
    unnamed_cols: Any = ...
    index_names: Any = ...
    col_names: Any = ...
    parse_dates: Any = ...
    date_parser: Any = ...
    dayfirst: Any = ...
    keep_date_col: Any = ...
    na_values: Any = ...
    na_fvalues: Any = ...
    na_filter: Any = ...
    keep_default_na: Any = ...
    true_values: Any = ...
    false_values: Any = ...
    mangle_dupe_cols: Any = ...
    infer_datetime_format: Any = ...
    cache_dates: Any = ...
    header: Any = ...
    handles: Any = ...
    def __init__(self, kwds: Any) -> None: ...
    def close(self) -> None: ...

class CParserWrapper(ParserBase):
    kwds: Any = ...
    unnamed_cols: Any = ...
    names: Any = ...
    orig_names: Any = ...
    index_names: Any = ...
    def __init__(self, src: Any, **kwds: Any) -> None: ...
    def close(self) -> None: ...
    def set_error_bad_lines(self, status: Any) -> None: ...
    def read(self, nrows: Optional[Any] = ...) -> Any: ...

def TextParser(*args: Any, **kwds: Any) -> Any: ...
def count_empty_vals(vals: Any) -> Any: ...

class PythonParser(ParserBase):
    data: Any = ...
    buf: Any = ...
    pos: int = ...
    line_pos: int = ...
    encoding: Any = ...
    compression: Any = ...
    memory_map: Any = ...
    skiprows: Any = ...
    skipfunc: Any = ...
    skipfooter: Any = ...
    delimiter: Any = ...
    quotechar: Any = ...
    escapechar: Any = ...
    doublequote: Any = ...
    skipinitialspace: Any = ...
    lineterminator: Any = ...
    quoting: Any = ...
    skip_blank_lines: Any = ...
    warn_bad_lines: Any = ...
    error_bad_lines: Any = ...
    names_passed: Any = ...
    has_index_names: bool = ...
    verbose: Any = ...
    converters: Any = ...
    dtype: Any = ...
    thousands: Any = ...
    decimal: Any = ...
    comment: Any = ...
    num_original_columns: Any = ...
    columns: Any = ...
    orig_names: Any = ...
    index_names: Any = ...
    nonnum: Any = ...
    def __init__(self, f: Any, **kwds: Any) -> None: ...
    def read(self, rows: Optional[Any] = ...) -> Any: ...
    def get_chunk(self, size: Optional[Any] = ...) -> Any: ...

class FixedWidthReader(abc.Iterator[Any]):
    f: Any = ...
    buffer: Any = ...
    delimiter: Any = ...
    comment: Any = ...
    colspecs: Any = ...
    def __init__(self, f: Any, colspecs: Any, delimiter: Any, comment: Any, skiprows: Optional[Any] = ..., infer_nrows: int = ...) -> None: ...
    def get_rows(self, infer_nrows: Any, skiprows: Optional[Any] = ...) -> Any: ...
    def detect_colspecs(self, infer_nrows: int = ..., skiprows: Optional[Any] = ...) -> Any: ...
    def __next__(self) -> Any: ...

class FixedWidthFieldParser(PythonParser):
    colspecs: Any = ...
    infer_nrows: Any = ...
    def __init__(self, f: Any, **kwds: Any) -> None: ...



from typing import Any

def create_pandas_abc_type(name: Any, attr: Any, comp: Any) -> Any: ...

ABCIndex: Any
ABCInt64Index: Any
ABCUInt64Index: Any
ABCRangeIndex: Any
ABCFloat64Index: Any
ABCMultiIndex: Any
ABCDatetimeIndex: Any
ABCTimedeltaIndex: Any
ABCPeriodIndex: Any
ABCCategoricalIndex: Any
ABCIntervalIndex: Any
ABCIndexClass: Any
ABCSeries: Any
ABCDataFrame: Any
ABCSparseArray: Any
ABCCategorical: Any
ABCDatetimeArray: Any
ABCTimedeltaArray: Any
ABCPeriodArray: Any
ABCPeriod: Any
ABCDateOffset: Any
ABCInterval: Any
ABCExtensionArray: Any
ABCPandasArray: Any

class _ABCGeneric(type):
    def __instancecheck__(cls: Any, inst: Any) -> bool: ...

ABCGeneric: Any



from pandas._typing import Axis as Axis, FrameOrSeries as FrameOrSeries, Scalar as Scalar
from pandas.core.base import DataError as DataError, PandasObject as PandasObject, SelectionMixin as SelectionMixin, ShallowMixin as ShallowMixin
from pandas.core.dtypes.common import ensure_float64 as ensure_float64, is_bool as is_bool, is_float_dtype as is_float_dtype, is_integer as is_integer, is_integer_dtype as is_integer_dtype, is_list_like as is_list_like, is_scalar as is_scalar, needs_i8_conversion as needs_i8_conversion
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCDateOffset as ABCDateOffset, ABCDatetimeIndex as ABCDatetimeIndex, ABCPeriodIndex as ABCPeriodIndex, ABCSeries as ABCSeries, ABCTimedeltaIndex as ABCTimedeltaIndex
from pandas.core.indexes.api import Index as Index, ensure_index as ensure_index
from pandas.core.window.common import WindowGroupByMixin as WindowGroupByMixin, calculate_center_offset as calculate_center_offset, calculate_min_periods as calculate_min_periods, get_weighted_roll_func as get_weighted_roll_func, zsqrt as zsqrt
from pandas.core.window.indexers import BaseIndexer as BaseIndexer, FixedWindowIndexer as FixedWindowIndexer, VariableWindowIndexer as VariableWindowIndexer
from pandas.core.window.numba_ import generate_numba_apply_func as generate_numba_apply_func
from pandas.util._decorators import Appender as Appender, Substitution as Substitution, cache_readonly as cache_readonly
from typing import Any, Dict, Optional, Set, Tuple, Union



class _Window(PandasObject, ShallowMixin, SelectionMixin):
    exclusions: Set[str] = ...
    obj: Any = ...
    on: Any = ...
    closed: Any = ...
    window: Any = ...
    min_periods: Any = ...
    center: Any = ...
    win_type: Any = ...
    win_freq: Any = ...
    axis: Any = ...
    def __init__(self, obj: Any, window: Any = ..., min_periods: Optional[int]=..., center: Optional[bool]=..., win_type: Optional[str]=..., axis: Axis=..., on: Optional[Union[str, Index]]=..., closed: Optional[str]=..., **kwargs: Any) -> None: ...
    @property
    def is_datetimelike(self) -> Optional[bool]: ...
    @property
    def is_freq_type(self) -> bool: ...
    def validate(self) -> None: ...
    def __getattr__(self, attr: str) -> Any: ...
    def __iter__(self) -> Any: ...
    def aggregate(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    agg: Any = ...

class Window(_Window):
    def validate(self) -> None: ...
    def aggregate(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    agg: Any = ...
    def sum(self, *args: Any, **kwargs: Any) -> Any: ...
    def mean(self, *args: Any, **kwargs: Any) -> Any: ...
    def var(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def std(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...

class _Rolling(_Window): ...

class _Rolling_and_Expanding(_Rolling):
    def count(self) -> Any: ...
    def apply(self, func: Any, raw: bool=..., engine: str=..., engine_kwargs: Optional[Dict[str, Any]]=..., args: Optional[Tuple[Any, ...]]=..., kwargs: Optional[Dict[str, Any]]=...) -> Any: ...
    def sum(self, *args: Any, **kwargs: Any) -> Any: ...
    def max(self, *args: Any, **kwargs: Any) -> Any: ...
    def min(self, *args: Any, **kwargs: Any) -> Any: ...
    def mean(self, *args: Any, **kwargs: Any) -> Any: ...
    def median(self, **kwargs: Any) -> Any: ...
    def std(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def var(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def skew(self, **kwargs: Any) -> Any: ...
    def kurt(self, **kwargs: Any) -> Any: ...
    def quantile(self, quantile: Any, interpolation: str = ..., **kwargs: Any) -> Any: ...
    def cov(self, other: Optional[Any] = ..., pairwise: Optional[Any] = ..., ddof: int = ..., **kwargs: Any) -> Any: ...
    def corr(self, other: Optional[Any] = ..., pairwise: Optional[Any] = ..., **kwargs: Any) -> Any: ...

class Rolling(_Rolling_and_Expanding):
    def is_datetimelike(self) -> bool: ...
    win_freq: Any = ...
    window: Any = ...
    win_type: str = ...
    min_periods: int = ...
    def validate(self) -> None: ...
    def aggregate(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    agg: Any = ...
    def count(self) -> Any: ...
    def apply(self, func: Any, raw: bool = ..., engine: str = ..., engine_kwargs: Optional[Any] = ..., args: Optional[Any] = ..., kwargs: Optional[Any] = ...) -> Any: ...
    def sum(self, *args: Any, **kwargs: Any) -> Any: ...
    def max(self, *args: Any, **kwargs: Any) -> Any: ...
    def min(self, *args: Any, **kwargs: Any) -> Any: ...
    def mean(self, *args: Any, **kwargs: Any) -> Any: ...
    def median(self, **kwargs: Any) -> Any: ...
    def std(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def var(self, ddof: int = ..., *args: Any, **kwargs: Any) -> Any: ...
    def skew(self, **kwargs: Any) -> Any: ...
    def kurt(self, **kwargs: Any) -> Any: ...
    def quantile(self, quantile: Any, interpolation: str = ..., **kwargs: Any) -> Any: ...
    def cov(self, other: Optional[Any] = ..., pairwise: Optional[Any] = ..., ddof: int = ..., **kwargs: Any) -> Any: ...
    def corr(self, other: Optional[Any] = ..., pairwise: Optional[Any] = ..., **kwargs: Any) -> Any: ...

class RollingGroupby(WindowGroupByMixin, Rolling): ...



from collections import namedtuple

from typing import Any

OutputKey = namedtuple('OutputKey', ['label', 'position'])

class GroupByMixin: ...

plotting_methods: Any
common_apply_whitelist: Any
series_apply_whitelist: Any
dataframe_apply_whitelist: Any
cythonized_kernels: Any
cython_cast_blacklist: Any
reduction_kernels: Any
transformation_kernels: Any
groupby_other_methods: Any
transform_kernel_whitelist: Any



from pandas._config import config as config
from pandas._config.config import describe_option as describe_option, get_option as get_option, option_context as option_context, options as options, reset_option as reset_option, set_option as set_option
from pandas._config.display import detect_console_encoding as detect_console_encoding



import numpy as np
from pandas._typing import Dtype as Dtype
from pandas.core import algorithms as algorithms
from pandas.core.dtypes.cast import astype_nansafe as astype_nansafe

from pandas.core.dtypes.generic import ABCFloat64Index as ABCFloat64Index, ABCInt64Index as ABCInt64Index, ABCRangeIndex as ABCRangeIndex, ABCSeries as ABCSeries, ABCUInt64Index as ABCUInt64Index
from pandas.core.dtypes.missing import isna as isna
from pandas.core.indexes.base import Index as Index, InvalidIndexError as InvalidIndexError, maybe_extract_name as maybe_extract_name
from pandas.core.ops import get_op_result_name as get_op_result_name
from pandas.util._decorators import Appender as Appender, cache_readonly as cache_readonly
from typing import Any, Optional

class NumericIndex(Index):
    def __new__(cls, data: Optional[Any] = ..., dtype: Optional[Any] = ..., copy: bool = ..., name: Optional[Any] = ...) -> Any: ...
    @property
    def is_all_dates(self) -> bool: ...
    def insert(self, loc: Any, item: Any) -> Any: ...

class IntegerIndex(NumericIndex):
    def __contains__(self, key: Any) -> bool: ...

class Int64Index(IntegerIndex):
    __doc__: Any = ...
    @property
    def inferred_type(self) -> str: ...
    @property
    def asi8(self) -> np.ndarray: ...

class UInt64Index(IntegerIndex):
    __doc__: Any = ...
    @property
    def inferred_type(self) -> str: ...
    @property
    def asi8(self) -> np.ndarray: ...

class Float64Index(NumericIndex):
    __doc__: Any = ...
    @property
    def inferred_type(self) -> str: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def get_value(self, series: Any, key: Any) -> Any: ...
    def equals(self, other: Any) -> bool: ...
    def __contains__(self, other: Any) -> bool: ...
    def get_loc(self, key: Any, method: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def is_unique(self) -> bool: ...
    def isin(self, values: Any, level: Optional[Any] = ...) -> Any: ...



from pandas._typing import FilePathOrBuffer as FilePathOrBuffer
from pandas.io.common import get_filepath_or_buffer as get_filepath_or_buffer, get_handle as get_handle
from typing import Any, Optional, AnyStr


def to_pickle(obj: Any, filepath_or_buffer: FilePathOrBuffer[AnyStr], compression: Optional[str]=..., protocol: int=...) -> Any: ...
def read_pickle(filepath_or_buffer: FilePathOrBuffer[AnyStr], compression: Optional[str]=...) -> Any: ...




from pandas.core.ops.roperator import rdivmod as rdivmod, rfloordiv as rfloordiv, rmod as rmod
from typing import Any

def fill_zeros(result: Any, x: Any, y: Any) -> Any: ...
def mask_zero_div_zero(x: Any, y: Any, result: Any) -> Any: ...
def dispatch_fill_zeros(op: Any, left: Any, right: Any, result: Any) -> Any: ...



from pandas.core.dtypes.generic import ABCMultiIndex as ABCMultiIndex
from pandas.io.formats.format import DataFrameFormatter as DataFrameFormatter, TableFormatter as TableFormatter
from typing import Any, IO, Optional

class LatexFormatter(TableFormatter):
    fmt: Any = ...
    frame: Any = ...
    bold_rows: Any = ...
    column_format: Any = ...
    longtable: Any = ...
    multicolumn: Any = ...
    multicolumn_format: Any = ...
    multirow: Any = ...
    caption: Any = ...
    label: Any = ...
    escape: Any = ...
    def __init__(self, formatter: DataFrameFormatter, column_format: Optional[str]=..., longtable: bool=..., multicolumn: bool=..., multicolumn_format: Optional[str]=..., multirow: bool=..., caption: Optional[str]=..., label: Optional[str]=...) -> None: ...
    clinebuf: Any = ...
    def write_result(self, buf: IO[str]) -> None: ...



from pandas.io.clipboards import read_clipboard as read_clipboard
from pandas.io.excel import ExcelFile as ExcelFile, ExcelWriter as ExcelWriter, read_excel as read_excel
from pandas.io.feather_format import read_feather as read_feather
from pandas.io.gbq import read_gbq as read_gbq
from pandas.io.html import read_html as read_html
from pandas.io.json import read_json as read_json
from pandas.io.orc import read_orc as read_orc
from pandas.io.parquet import read_parquet as read_parquet
from pandas.io.parsers import read_csv as read_csv, read_fwf as read_fwf, read_table as read_table
from pandas.io.pickle import read_pickle as read_pickle, to_pickle as to_pickle
from pandas.io.pytables import HDFStore as HDFStore, read_hdf as read_hdf
from pandas.io.sas import read_sas as read_sas
from pandas.io.spss import read_spss as read_spss
from pandas.io.sql import read_sql as read_sql, read_sql_query as read_sql_query, read_sql_table as read_sql_table
from pandas.io.stata import read_stata as read_stata



import pandas as pd
import pandas.core.common as com
from pandas._libs.tslibs import Timedelta as Timedelta, Timestamp as Timestamp
from pandas.core.computation import expr as expr, ops as ops, scope as _scope
from pandas.core.computation.expr import BaseExprVisitor as BaseExprVisitor
from pandas.core.computation.ops import UndefinedVariableError as UndefinedVariableError, is_term as is_term

from pandas.io.formats.printing import pprint_thing as pprint_thing, pprint_thing_encoded as pprint_thing_encoded
from typing import Any, Dict, Optional, Tuple

class PyTablesScope(_scope.Scope):
    queryables: Dict[str, Any]
    def __init__(self, level: int, global_dict: Any = ..., local_dict: Any = ..., queryables: Optional[Dict[str, Any]]=...) -> None: ...

class Term(ops.Term):
    env: PyTablesScope
    def __new__(cls, name: Any, env: Any, side: Optional[Any] = ..., encoding: Optional[Any] = ...) -> Any: ...
    def __init__(self, name: Any, env: PyTablesScope, side: Any = ..., encoding: Any = ...) -> None: ...
    @property
    def value(self) -> Any: ...
    @value.setter
    def value(self, new_value: Any) -> None: ...

class Constant(Term):
    def __init__(self, value: Any, env: PyTablesScope, side: Any = ..., encoding: Any = ...) -> None: ...

class BinOp(ops.BinOp):
    op: str
    queryables: Dict[str, Any]
    encoding: Any = ...
    condition: Any = ...
    def __init__(self, op: str, lhs: Any, rhs: Any, queryables: Dict[str, Any], encoding: Any) -> None: ...
    def prune(self, klass: Any) -> Any: ...
    def conform(self, rhs: Any) -> Any: ...
    @property
    def is_valid(self) -> bool: ...
    @property
    def is_in_table(self) -> bool: ...
    @property
    def kind(self) -> Any: ...
    @property
    def meta(self) -> Any: ...
    @property
    def metadata(self) -> Any: ...
    def generate(self, v: Any) -> str: ...
    def convert_value(self, v: Any) -> TermValue: ...
    def convert_values(self) -> None: ...

class FilterBinOp(BinOp):
    filter: Optional[Tuple[Any, Any, pd.Index]] = ...
    def invert(self) -> Any: ...
    def format(self) -> Any: ...
    # The idea behind this override in unclear
    def evaluate(self) -> Any: ...  # type: ignore
    def generate_filter_op(self, invert: bool=...) -> Any: ...

class JointFilterBinOp(FilterBinOp):
    def format(self) -> None: ...
    def evaluate(self) -> Any: ...  # type: ignore

class ConditionBinOp(BinOp):
    def invert(self) -> None: ...
    def format(self) -> Any: ...
    condition: Any = ...
    # The idea behind this override in unclear
    def evaluate(self) -> Any: ...  # type: ignore

class JointConditionBinOp(ConditionBinOp):
    condition: Any = ...
    def evaluate(self) -> Any: ...  # type: ignore

class UnaryOp(ops.UnaryOp):
    def prune(self, klass: Any) -> Any: ...

class PyTablesExprVisitor(BaseExprVisitor):
    const_type: Any = ...
    term_type: Any = ...
    def __init__(self, env: Any, engine: Any, parser: Any, **kwargs: Any) -> None: ...
    def visit_UnaryOp(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Index(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Assign(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Subscript(self, node: Any, **kwargs: Any) -> Any: ...
    def visit_Attribute(self, node: Any, **kwargs: Any) -> Any: ...
    def translate_In(self, op: Any) -> Any: ...

class PyTablesExpr(expr.Expr):
    env: PyTablesScope
    encoding: Any = ...
    condition: Any = ...
    filter: Any = ...
    terms: Any = ...
    expr: Any = ...
    def __init__(self, where: Any, queryables: Optional[Dict[str, Any]]=..., encoding: Any = ..., scope_level: int=...) -> None: ...
    def evaluate(self) -> Any: ...

class TermValue:
    value: Any = ...
    converted: Any = ...
    kind: Any = ...
    def __init__(self, value: Any, converted: Any, kind: str) -> None: ...
    def tostring(self, encoding: Any) -> str: ...

def maybe_expression(s: Any) -> bool: ...



# Removed some imports that were necessary only for implementation
import numpy as np
from pandas._libs.tslibs.period import Period as Period
from pandas._libs.tslibs.timedeltas import Timedelta as Timedelta
from pandas.core.arrays import datetimelike as dtl
from pandas.core.dtypes.dtypes import PeriodDtype as PeriodDtype
from pandas.core.dtypes.generic import ABCIndexClass as ABCIndexClass, ABCPeriodArray as ABCPeriodArray, ABCPeriodIndex as ABCPeriodIndex, ABCSeries as ABCSeries
from pandas.tseries import frequencies as frequencies
from pandas.tseries.offsets import DateOffset as DateOffset, Tick as Tick
from typing import Any, Optional, Sequence, Union

class PeriodArray(dtl.DatetimeLikeArrayMixin, dtl.DatelikeOps):
    __array_priority__: int = ...
    def __init__(self, values: Any, freq: Optional[Any] = ..., dtype: Optional[Any] = ..., copy: bool = ...) -> None: ...
    def dtype(self) -> Any: ...
    @property  # type: ignore
    def freq(self) -> Any: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __arrow_array__(self, type: Optional[Any] = ...) -> Any: ...
    year: Any = ...
    month: Any = ...
    day: Any = ...
    hour: Any = ...
    minute: Any = ...
    second: Any = ...
    weekofyear: Any = ...
    week: Any = ...
    dayofweek: Any = ...
    weekday: Any = ...
    dayofyear: Any = ...
    day_of_year: Any = ...
    quarter: Any = ...
    qyear: Any = ...
    days_in_month: Any = ...
    daysinmonth: Any = ...
    @property
    def is_leap_year(self) -> Any: ...
    @property
    def start_time(self) -> Any: ...
    @property
    def end_time(self) -> Any: ...
    def to_timestamp(self, freq: Optional[Any] = ..., how: str = ...) -> Any: ...
    def asfreq(self, freq: Optional[Any] = ..., how: str = ...) -> Any: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...

def raise_on_incompatible(left: Any, right: Any) -> Any: ...
def period_array(data: Sequence[Optional[Period]], freq: Optional[Union[str, Tick]]=..., copy: bool=...) -> PeriodArray: ...
def validate_dtype_freq(dtype: Any, freq: Any) -> Any: ...
def dt64arr_to_periodarr(data: Any, freq: Any, tz: Optional[Any] = ...) -> Any: ...



# Removed some imports that were necessary only for implementation
import numpy as np
from pandas._typing import AnyArrayLike as AnyArrayLike, GeneralDuplicatesKeepStrategy
from pandas.core import accessor as accessor
from pandas.core.algorithms import take_1d as take_1d
from pandas.core.arrays.categorical import Categorical as Categorical, contains as contains
from pandas.core.dtypes.dtypes import CategoricalDtype as CategoricalDtype
from pandas.core.dtypes.generic import ABCCategorical as ABCCategorical, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from pandas.core.indexes.base import Index as Index, maybe_extract_name as maybe_extract_name
from pandas.core.indexes.extension import ExtensionIndex as ExtensionIndex, inherit_names as inherit_names
from pandas.core.ops import get_op_result_name as get_op_result_name
from pandas.util._decorators import Appender as Appender, cache_readonly as cache_readonly
from typing import Any, Optional

class CategoricalIndex(ExtensionIndex, accessor.PandasDelegate):
    codes: np.ndarray
    categories: Index
    def __new__(cls, data: Optional[Any] = ..., categories: Optional[Any] = ..., ordered: Optional[Any] = ..., dtype: Optional[Any] = ..., copy: bool = ..., name: Optional[Any] = ...) -> Any: ...
    def equals(self, other: Any) -> Any: ...
    @property
    def inferred_type(self) -> str: ...
    @property
    def values(self) -> Any: ...
    def __contains__(self, key: Any) -> bool: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def fillna(self, value: Any, downcast: Optional[Any] = ...) -> Any: ... # type: ignore
    def is_unique(self) -> bool: ...
    @property
    def is_monotonic_increasing(self) -> Any: ...
    @property
    def is_monotonic_decreasing(self) -> bool: ...
    def unique(self, level: Optional[Any] = ...) -> Any: ...
    def duplicated(self, keep: GeneralDuplicatesKeepStrategy = ...) -> Any: ...
    def get_loc(self, key: Any, method: Optional[Any] = ...) -> Any: ... # type: ignore
    def get_value(self, series: AnyArrayLike, key: Any) -> Any: ...
    def where(self, cond: Any, other: Optional[Any] = ...) -> Any: ...
    def reindex(self, target: Any, method: Optional[Any] = ..., level: Optional[Any] = ..., limit: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def get_indexer(self, target: Any, method: Optional[Any] = ..., limit: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def get_indexer_non_unique(self, target: Any) -> Any: ...
    def take_nd(self, *args: Any, **kwargs: Any) -> Any: ...
    def map(self, mapper: Any) -> Any: ...  # type: ignore
    def delete(self, loc: Any) -> Any: ...
    def insert(self, loc: Any, item: Any) -> Any: ...




from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCSeries as ABCSeries
from pandas.core.groupby.base import GroupByMixin as GroupByMixin
from pandas.core.indexes.api import MultiIndex as MultiIndex
from typing import Any, Callable, Optional, Sequence


class WindowGroupByMixin(GroupByMixin):
    def __init__(self, obj: Any, *args: Any, **kwargs: Any) -> None: ...
    count: Any = ...
    corr: Any = ...
    cov: Any = ...

def calculate_center_offset(window: Any) -> Any: ...
def calculate_min_periods(window: int, min_periods: Optional[int], num_values: int, required_min_periods: int, floor: int) -> int: ...
def zsqrt(x: Any) -> Any: ...
def prep_binary(arg1: Any, arg2: Any) -> Any: ...
def get_weighted_roll_func(cfunc: Callable[..., Any]) -> Callable[..., Any]: ...



from pandas._libs.interval import Interval as Interval
from pandas._libs.tslibs import  Period as Period, Timestamp as Timestamp, timezones as timezones
from pandas._typing import Ordered as Ordered
from pandas.core.dtypes.base import ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.generic import ABCCategoricalIndex as ABCCategoricalIndex, ABCDateOffset as ABCDateOffset, ABCIndexClass as ABCIndexClass
from pandas.core.dtypes.inference import is_bool as is_bool, is_list_like as is_list_like
from typing import Any, Optional, Tuple, Type, Union

str_type = str

def register_extension_dtype(cls: Type[ExtensionDtype]) -> Type[ExtensionDtype]: ...

class Registry:
    dtypes: Any = ...
    def __init__(self) -> None: ...
    def register(self, dtype: Type[ExtensionDtype]) -> None: ...
    def find(self, dtype: Union[Type[ExtensionDtype], str]) -> Optional[Type[ExtensionDtype]]: ...

registry: Any

class PandasExtensionDtype(ExtensionDtype):
    type: Any
    kind: Any
    subdtype: Any = ...
    str: Optional[str_type] = ...
    num: int = ...
    shape: Tuple[int, ...] = ...
    itemsize: int = ...
    base: Any = ...
    isbuiltin: int = ...
    isnative: int = ...
    def __hash__(self) -> int: ...
    @classmethod
    def reset_cache(cls: Any) -> None: ...

class CategoricalDtypeType(type): ...

class CategoricalDtype(PandasExtensionDtype, ExtensionDtype):
    name: str = ...
    type: Type[CategoricalDtypeType] = ...
    kind: str_type = ...
    str: str = ...
    base: Any = ...
    def __init__(self, categories: Any = ..., ordered: Ordered=...) -> None: ...
    @classmethod
    def construct_from_string(cls: Any, string: str_type) -> CategoricalDtype: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: Any) -> bool: ...
    @classmethod
    def construct_array_type(cls) -> Any: ...
    @staticmethod
    def validate_ordered(ordered: Ordered) -> None: ...
    @staticmethod
    def validate_categories(categories: Any, fastpath: bool=...) -> Any: ...
    def update_dtype(self, dtype: Union[str_type, CategoricalDtype]) -> CategoricalDtype: ...
    @property
    def categories(self) -> Any: ...
    @property
    def ordered(self) -> Ordered: ...

class DatetimeTZDtype(PandasExtensionDtype):
    type: Type[Timestamp] = ...
    kind: str_type = ...
    str: str = ...
    num: int = ...
    base: Any = ...
    na_value: Any = ...
    def __init__(self, unit: str_type = ..., tz: Optional[Any] = ...) -> None: ...
    @property
    def unit(self) -> Any: ...
    @property
    def tz(self) -> Any: ...
    @classmethod
    def construct_array_type(cls) -> Any: ...
    @classmethod
    def construct_from_string(cls: Any, string: str_type) -> Any: ...
    @property
    def name(self) -> str_type: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: Any) -> bool: ...

class PeriodDtype(PandasExtensionDtype):
    type: Type[Period] = ...
    kind: str_type = ...
    str: str = ...
    base: Any = ...
    num: int = ...
    def __new__(cls, freq: Optional[Any] = ...) -> Any: ...
    @property
    def freq(self) -> Any: ...
    @classmethod
    def construct_from_string(cls, string: Any) -> Any: ...
    @property
    def name(self) -> str_type: ...
    @property
    def na_value(self) -> Any: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: Any) -> bool: ...
    @classmethod
    def is_dtype(cls: Any, dtype: Any) -> bool: ...
    @classmethod
    def construct_array_type(cls) -> Any: ...
    def __from_arrow__(self, array: Any) -> Any: ...

class IntervalDtype(PandasExtensionDtype):
    name: str = ...
    kind: str_type = ...
    str: str = ...
    base: Any = ...
    num: int = ...
    def __new__(cls, subtype: Optional[Any] = ...) -> Any: ...
    @property
    def subtype(self) -> Any: ...
    @classmethod
    def construct_array_type(cls) -> Any: ...
    @classmethod
    def construct_from_string(cls, string: Any) -> Any: ...
    @property
    def type(self) -> Any: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: Any) -> bool: ...
    @classmethod
    def is_dtype(cls: Any, dtype: Any) -> bool: ...
    def __from_arrow__(self, array: Any) -> Any: ...



from pandas._config.config import is_bool as is_bool, is_callable as is_callable, is_instance_factory as is_instance_factory, is_int as is_int, is_nonnegative_int as is_nonnegative_int, is_one_of_factory as is_one_of_factory, is_text as is_text
from typing import Any

use_bottleneck_doc: str

def use_bottleneck_cb(key: Any) -> None: ...

use_numexpr_doc: str

def use_numexpr_cb(key: Any) -> None: ...

pc_precision_doc: str
pc_colspace_doc: str
pc_max_rows_doc: str
pc_min_rows_doc: str
pc_max_cols_doc: str
pc_max_categories_doc: str
pc_max_info_cols_doc: str
pc_nb_repr_h_doc: str
pc_pprint_nest_depth: str
pc_multi_sparse_doc: str
float_format_doc: str
max_colwidth_doc: str
colheader_justify_doc: str
pc_expand_repr_doc: str
pc_show_dimensions_doc: str
pc_east_asian_width_doc: str
pc_ambiguous_as_wide_doc: str
pc_latex_repr_doc: str
pc_table_schema_doc: str
pc_html_border_doc: str
pc_html_use_mathjax_doc: str
pc_width_doc: str
pc_chop_threshold_doc: str
pc_max_seq_items: str
pc_max_info_rows_doc: str
pc_large_repr_doc: str
pc_memory_usage_doc: str
pc_latex_escape: str
pc_latex_longtable: str
pc_latex_multicolumn: str
pc_latex_multicolumn_format: str
pc_latex_multirow: str

def table_schema_cb(key: Any) -> None: ...
def is_terminal() -> bool: ...

max_cols: int
tc_sim_interactive_doc: str
use_inf_as_null_doc: str
use_inf_as_na_doc: str

def use_inf_as_na_cb(key: Any) -> None: ...

chained_assignment: str
reader_engine_doc: str
writer_engine_doc: str
parquet_engine_doc: str
plotting_backend_doc: str

def register_plotting_backend_cb(key: Any) -> None: ...

register_converter_doc: str

def register_converter_cb(key: Any) -> None: ...



from pandas.plotting._matplotlib.boxplot import boxplot as boxplot, boxplot_frame as boxplot_frame, boxplot_frame_groupby as boxplot_frame_groupby
from pandas.plotting._matplotlib.converter import deregister as deregister, register as register
from pandas.plotting._matplotlib.hist import hist_frame as hist_frame, hist_series as hist_series
from pandas.plotting._matplotlib.misc import andrews_curves as andrews_curves, autocorrelation_plot as autocorrelation_plot, bootstrap_plot as bootstrap_plot, lag_plot as lag_plot, parallel_coordinates as parallel_coordinates, radviz as radviz, scatter_matrix as scatter_matrix
from pandas.plotting._matplotlib.tools import table as table
from typing import Any

def plot(data: Any, kind: Any, **kwargs: Any) -> Any: ...



from pandas.core.api import DataFrame as DataFrame
from pandas.core.dtypes.inference import is_list_like as is_list_like
from pathlib import Path
from typing import Optional, Sequence, Union

def read_spss(path: Union[str, Path], usecols: Optional[Sequence[str]]=..., convert_categoricals: bool=...) -> DataFrame: ...



from pandas.io.excel._base import ExcelWriter as ExcelWriter
from typing import Any, Optional

class _XlwtWriter(ExcelWriter):
    engine: str = ...
    supported_extensions: Any = ...
    book: Any = ...
    fm_datetime: Any = ...
    fm_date: Any = ...
    def __init__(self, path: Any, engine: Optional[Any] = ..., encoding: Optional[Any] = ..., mode: str = ..., **engine_kwargs: Any) -> None: ...
    def save(self) -> Any: ...
    def write_cells(self, cells: Any, sheet_name: Optional[Any] = ..., startrow: int = ..., startcol: int = ..., freeze_panes: Optional[Any] = ...) -> None: ...



import numpy as np
from pandas._typing import Scalar as Scalar
from pandas.core.algorithms import take as take
from pandas.core.arrays import ExtensionArray as ExtensionArray, ExtensionOpsMixin as ExtensionOpsMixin

from pandas.core.dtypes.missing import isna as isna, notna as notna

from typing import Any, Optional

class BaseMaskedArray(ExtensionArray, ExtensionOpsMixin):
    def __getitem__(self, item: Any) -> Any: ...
    def __iter__(self) -> Any: ...
    def __len__(self) -> int: ...
    def __invert__(self) -> Any: ...
    def to_numpy(self, dtype: Any = ..., copy: Any = ..., na_value: Scalar=...) -> Any: ...
    __array_priority__: int = ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    def __arrow_array__(self, type: Optional[Any] = ...) -> Any: ...
    def isna(self) -> Any: ...
    @property
    def nbytes(self) -> Any: ...
    def take(self, indexer: Any, allow_fill: bool = ..., fill_value: Optional[Any] = ...) -> Any: ...
    def copy(self) -> Any: ...
    def value_counts(self, dropna: bool = ...) -> Any: ...



import numpy as np
from pandas._typing import ArrayLike as ArrayLike
from pandas.core.dtypes.dtypes import CategoricalDtype as CategoricalDtype, DatetimeTZDtype as DatetimeTZDtype, ExtensionDtype as ExtensionDtype, IntervalDtype as IntervalDtype, PeriodDtype as PeriodDtype, registry as registry
from pandas.core.dtypes.generic import ABCCategorical as ABCCategorical, ABCDatetimeIndex as ABCDatetimeIndex, ABCIndexClass as ABCIndexClass, ABCPeriodArray as ABCPeriodArray, ABCPeriodIndex as ABCPeriodIndex, ABCSeries as ABCSeries
from pandas.core.dtypes.inference import is_array_like as is_array_like, is_bool as is_bool, is_complex as is_complex, is_decimal as is_decimal, is_dict_like as is_dict_like, is_file_like as is_file_like, is_float as is_float, is_hashable as is_hashable, is_integer as is_integer, is_interval as is_interval, is_iterator as is_iterator, is_list_like as is_list_like, is_named_tuple as is_named_tuple, is_nested_list_like as is_nested_list_like, is_number as is_number, is_re as is_re, is_re_compilable as is_re_compilable, is_scalar as is_scalar, is_sequence as is_sequence
from typing import Any, Callable, Union

ensure_float64: Any
ensure_float32: Any

def ensure_float(arr: Any) -> Any: ...

ensure_uint64: Any
ensure_int64: Any
ensure_int32: Any
ensure_int16: Any
ensure_int8: Any
ensure_platform_int: Any
ensure_object: Any

def ensure_str(value: Union[bytes, Any]) -> str: ...
def ensure_categorical(arr: Any) -> Any: ...
def ensure_int_or_float(arr: ArrayLike, copy: bool=...) -> np.array: ...
def ensure_python_int(value: Union[int, np.integer]) -> int: ...
def classes(*klasses: Any) -> Callable[..., Any]: ...
def classes_and_not_datetimelike(*klasses: Any) -> Callable[..., Any]: ...
def is_object_dtype(arr_or_dtype: Any) -> bool: ...
def is_sparse(arr: Any) -> bool: ...
def is_scipy_sparse(arr: Any) -> bool: ...
def is_categorical(arr: Any) -> bool: ...
def is_datetime64_dtype(arr_or_dtype: Any) -> bool: ...
def is_datetime64tz_dtype(arr_or_dtype: Any) -> bool: ...
def is_timedelta64_dtype(arr_or_dtype: Any) -> bool: ...
def is_period_dtype(arr_or_dtype: Any) -> bool: ...
def is_interval_dtype(arr_or_dtype: Any) -> bool: ...
def is_categorical_dtype(arr_or_dtype: Any) -> bool: ...
def is_string_dtype(arr_or_dtype: Any) -> bool: ...
def is_period_arraylike(arr: Any) -> bool: ...
def is_datetime_arraylike(arr: Any) -> bool: ...
def is_dtype_equal(source: Any, target: Any) -> bool: ...
def is_any_int_dtype(arr_or_dtype: Any) -> bool: ...
def is_integer_dtype(arr_or_dtype: Any) -> bool: ...
def is_signed_integer_dtype(arr_or_dtype: Any) -> bool: ...
def is_unsigned_integer_dtype(arr_or_dtype: Any) -> bool: ...
def is_int64_dtype(arr_or_dtype: Any) -> bool: ...
def is_datetime64_any_dtype(arr_or_dtype: Any) -> bool: ...
def is_datetime64_ns_dtype(arr_or_dtype: Any) -> bool: ...
def is_timedelta64_ns_dtype(arr_or_dtype: Any) -> bool: ...
def is_datetime_or_timedelta_dtype(arr_or_dtype: Any) -> bool: ...
def is_numeric_v_string_like(a: Any, b: Any) -> Any: ...
def is_datetimelike_v_numeric(a: Any, b: Any) -> Any: ...
def needs_i8_conversion(arr_or_dtype: Any) -> bool: ...
def is_numeric_dtype(arr_or_dtype: Any) -> bool: ...
def is_string_like_dtype(arr_or_dtype: Any) -> bool: ...
def is_float_dtype(arr_or_dtype: Any) -> bool: ...
def is_bool_dtype(arr_or_dtype: Any) -> bool: ...
def is_extension_type(arr: Any) -> bool: ...
def is_extension_array_dtype(arr_or_dtype: Any) -> bool: ...
def is_complex_dtype(arr_or_dtype: Any) -> bool: ...
def infer_dtype_from_object(dtype: Any) -> Any: ...
def pandas_dtype(dtype: Any) -> Any: ...



from pandas._typing import Dtype as Dtype
from pandas.core.dtypes.base import ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.cast import astype_nansafe as astype_nansafe

from pandas.core.dtypes.dtypes import register_extension_dtype as register_extension_dtype
from pandas.core.dtypes.missing import isna as isna, na_value_for_dtype as na_value_for_dtype
from typing import Any

class SparseDtype(ExtensionDtype):
    def __init__(self, dtype: Dtype=..., fill_value: Any = ...) -> None: ...
    def __hash__(self) -> Any: ...
    def __eq__(self, other: Any) -> bool: ...
    @property
    def fill_value(self) -> Any: ...
    @property
    def kind(self) -> Any: ...
    @property
    def type(self) -> Any: ...
    @property
    def subtype(self) -> Any: ...
    @property
    def name(self) -> Any: ...
    @classmethod
    def construct_array_type(cls) -> Any: ...
    @classmethod
    def construct_from_string(cls, string: Any) -> Any: ...
    @classmethod
    def is_dtype(cls, dtype: Any) -> Any: ...
    def update_dtype(self, dtype: Any) -> Any: ...



from pandas._typing import T as T

from typing import Any, Collection, Iterable, Optional, Union, Callable, Tuple, TypeVar, overload

class SettingWithCopyError(ValueError): ...
class SettingWithCopyWarning(Warning): ...

def flatten(l: Any) -> None: ...
def consensus_name_attr(objs: Any) -> Any: ...
def maybe_box(indexer: Any, values: Any, obj: Any, key: Any) -> Any: ...
def maybe_box_datetimelike(value: Any) -> Any: ...

values_from_object: Any

def is_bool_indexer(key: Any) -> bool: ...
def cast_scalar_indexer(val: Any) -> Any: ...
def not_none(*args: Any) -> Any: ...
def any_none(*args: Any) -> Any: ...
def all_none(*args: Any) -> Any: ...
def any_not_none(*args: Any) -> Any: ...
def all_not_none(*args: Any) -> Any: ...
def count_not_none(*args: Any) -> Any: ...
def try_sort(iterable: Any) -> Any: ...
def asarray_tuplesafe(values: Any, dtype: Optional[Any] = ...) -> Any: ...
def index_labels_to_array(labels: Any, dtype: Optional[Any] = ...) -> Any: ...
def maybe_make_list(obj: Any) -> Any: ...
def maybe_iterable_to_list(obj: Union[Iterable[T], T]) -> Union[Collection[T], T]: ...
def is_null_slice(obj: Any) -> Any: ...
def is_true_slices(l: Any) -> Any: ...
def is_full_slice(obj: Any, l: Any) -> Any: ...
def get_callable_name(obj: Any) -> Any: ...
def apply_if_callable(maybe_callable: Any, obj: Any, **kwargs: Any) -> Any: ...
def dict_compat(d: Any) -> Any: ...
def standardize_mapping(into: Any) -> Any: ...
def random_state(state: Optional[Any] = ...) -> Any: ...
def get_rename_function(mapper: Any) -> Any: ...

PipeReturn = TypeVar("PipeReturn")
@overload
def pipe(obj: Any, func: Union[Callable[..., PipeReturn], Tuple[Callable[..., PipeReturn], str]], *args: Any, **kwargs: Any) -> PipeReturn: ...
@overload
def pipe(obj: Any, func: PipeReturn, *args: Any, **kwargs: Any) -> PipeReturn: ...



from pandas.core.reshape.concat import concat as concat
from pandas.core.reshape.melt import lreshape as lreshape, melt as melt, wide_to_long as wide_to_long
from pandas.core.reshape.merge import merge as merge, merge_asof as merge_asof, merge_ordered as merge_ordered
from pandas.core.reshape.pivot import crosstab as crosstab, pivot as pivot, pivot_table as pivot_table
from pandas.core.reshape.reshape import get_dummies as get_dummies
from pandas.core.reshape.tile import cut as cut, qcut as qcut



import numpy as np
from pandas.core import ops as ops
from pandas.core.construction import extract_array as extract_array
from pandas.core.dtypes.common import ensure_platform_int as ensure_platform_int, ensure_python_int as ensure_python_int, is_integer as is_integer, is_integer_dtype as is_integer_dtype, is_list_like as is_list_like, is_scalar as is_scalar, is_timedelta64_dtype as is_timedelta64_dtype
from pandas.core.dtypes.generic import ABCTimedeltaIndex as ABCTimedeltaIndex
from pandas.core.indexes.base import Index as Index, maybe_extract_name as maybe_extract_name
from pandas.core.indexes.numeric import Int64Index as Int64Index
from pandas.core.ops.common import unpack_zerodim_and_defer as unpack_zerodim_and_defer
from pandas.io.formats.printing import pprint_thing as pprint_thing
from pandas.util._decorators import Appender as Appender, cache_readonly as cache_readonly
from typing import Any, Optional, Union

class RangeIndex(Int64Index):
    def __new__(cls, start: Optional[Any] = ..., stop: Optional[Any] = ..., step: Optional[Any] = ..., dtype: Optional[Any] = ..., copy: bool = ..., name: Optional[Any] = ...) -> Any: ...
    @classmethod
    def from_range(cls, data: Any, name: Optional[Any] = ..., dtype: Optional[Any] = ...) -> Any: ...
    def __reduce__(self) -> Any: ...
    def start(self) -> Any: ...
    def stop(self) -> Any: ...
    def step(self) -> Any: ...
    def nbytes(self) -> int: ...
    def memory_usage(self, deep: bool=...) -> int: ...
    @property
    def dtype(self) -> np.dtype: ...
    @property
    def is_unique(self) -> bool: ...
    def is_monotonic_increasing(self) -> bool: ...
    def is_monotonic_decreasing(self) -> bool: ...
    @property
    def has_duplicates(self) -> bool: ...
    def __contains__(self, key: Union[int, np.integer]) -> bool: ...
    def get_loc(self, key: Any, method: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def get_indexer(self, target: Any, method: Optional[Any] = ..., limit: Optional[Any] = ..., tolerance: Optional[Any] = ...) -> Any: ...
    def tolist(self) -> Any: ...
    def copy(self, name: Optional[Any] = ..., deep: bool = ..., dtype: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def min(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def max(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def argsort(self, *args: Any, **kwargs: Any) -> Any: ...
    def equals(self, other: Any) -> Any: ...
    def intersection(self, other: Any, sort: bool = ...) -> Any: ...
    def join(self, other: Any, how: str = ..., level: Optional[Any] = ..., return_indexers: bool = ..., sort: bool = ...) -> Any: ...
    def __len__(self) -> int: ...
    @property
    def size(self) -> int: ...
    def __getitem__(self, key: Any) -> Any: ...
    def __floordiv__(self, other: Any) -> Any: ...
    def all(self) -> bool: ...
    def any(self) -> bool: ...



# Removed some imports that were necessary only for implementation
import numpy as np
from pandas._libs import  NaTType as NaTType
from pandas._typing import DatetimeLikeScalar as DatetimeLikeScalar
from pandas.core import missing as missing, nanops as nanops, ops as ops
from pandas.core.algorithms import checked_add_with_arr as checked_add_with_arr, take as take, unique1d as unique1d, value_counts as value_counts
from pandas.core.arrays.base import ExtensionArray as ExtensionArray, ExtensionOpsMixin as ExtensionOpsMixin

from pandas.core.dtypes.generic import ABCSeries as ABCSeries
from pandas.core.dtypes.inference import is_array_like as is_array_like
from pandas.core.dtypes.missing import is_valid_nat_for_dtype as is_valid_nat_for_dtype, isna as isna

from pandas.core.ops.common import unpack_zerodim_and_defer as unpack_zerodim_and_defer
from pandas.core.ops.invalid import invalid_comparison as invalid_comparison, make_invalid_op as make_invalid_op
from pandas.tseries import frequencies as frequencies
from pandas.tseries.offsets import DateOffset as DateOffset, Tick as Tick
from pandas.util._decorators import Appender as Appender, Substitution as Substitution
from pandas.util._validators import validate_fillna_kwargs as validate_fillna_kwargs
from typing import Any, Optional, Sequence, Union

class AttributesMixin: ...

class DatelikeOps:
    def strftime(self, date_format: Any) -> Any: ...

class TimelikeOps:
    def round(self, freq: Any, ambiguous: str = ..., nonexistent: str = ...) -> Any: ...
    def floor(self, freq: Any, ambiguous: str = ..., nonexistent: str = ...) -> Any: ...
    def ceil(self, freq: Any, ambiguous: str = ..., nonexistent: str = ...) -> Any: ...

class DatetimeLikeArrayMixin(ExtensionOpsMixin, AttributesMixin, ExtensionArray):
    @property
    def ndim(self) -> int: ...
    @property
    def shape(self) -> Any: ...
    def reshape(self, *args: Any, **kwargs: Any) -> Any: ...
    def ravel(self, *args: Any, **kwargs: Any) -> Any: ...
    def __iter__(self) -> Any: ...
    @property
    def asi8(self) -> np.ndarray: ...
    @property
    def nbytes(self) -> Any: ...
    def __array__(self, dtype: Any = ...) -> np.ndarray: ...
    @property
    def size(self) -> int: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: Any) -> Any: ...
    def __setitem__(self, key: Union[int, Sequence[int], Sequence[bool], slice], value: Union[NaTType, Any, Sequence[Any]]) -> None: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def view(self, dtype: Optional[Any] = ...) -> Any: ...
    def unique(self) -> Any: ...
    def take(self, indices: Any, allow_fill: bool = ..., fill_value: Optional[Any] = ...) -> Any: ...
    def copy(self) -> Any: ...
    def shift(self, periods: int = ..., fill_value: Optional[Any] = ..., axis: int = ...) -> Any: ...
    def searchsorted(self, value: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
    def repeat(self, repeats: Any, *args: Any, **kwargs: Any) -> Any: ...
    def value_counts(self, dropna: bool = ...) -> Any: ...
    def map(self, mapper: Any) -> Any: ...
    def isna(self) -> Any: ...
    def fillna(self, value: Optional[Any] = ..., method: Optional[Any] = ..., limit: Optional[Any] = ...) -> Any: ...
    @property
    def freq(self) -> Any: ...
    @freq.setter
    def freq(self, value: Any) -> None: ...
    @property
    def freqstr(self) -> Any: ...
    @property
    def inferred_freq(self) -> Any: ...
    @property
    def resolution(self) -> Any: ...
    __pow__: Any = ...
    __rpow__: Any = ...
    __mul__: Any = ...
    __rmul__: Any = ...
    __truediv__: Any = ...
    __rtruediv__: Any = ...
    __floordiv__: Any = ...
    __rfloordiv__: Any = ...
    __mod__: Any = ...
    __rmod__: Any = ...
    __divmod__: Any = ...
    __rdivmod__: Any = ...
    def __add__(self, other: Any) -> Any: ...
    def __radd__(self, other: Any) -> Any: ...
    def __sub__(self, other: Any) -> Any: ...
    def __rsub__(self, other: Any) -> Any: ...
    def __iadd__(self, other: Any) -> Any: ...
    def __isub__(self, other: Any) -> Any: ...
    def min(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def max(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def mean(self, skipna: bool = ...) -> Any: ...

def validate_periods(periods: Any) -> Any: ...
def validate_endpoints(closed: Any) -> Any: ...
def validate_inferred_freq(freq: Any, inferred_freq: Any, freq_infer: Any) -> Any: ...
def maybe_infer_freq(freq: Any) -> Any: ...



from pandas._typing import FilePathOrBuffer as FilePathOrBuffer, Scalar as Scalar
from pandas.io.excel._base import ExcelWriter as ExcelWriter, _BaseExcelReader
from typing import Any, List, Optional, AnyStr


class _OpenpyxlWriter(ExcelWriter):
    engine: str = ...
    supported_extensions: Any = ...
    book: Any = ...
    def __init__(self, path: Any, engine: Optional[Any] = ..., mode: str = ..., **engine_kwargs: Any) -> None: ...
    def save(self) -> Any: ...
    def write_cells(self, cells: Any, sheet_name: Optional[Any] = ..., startrow: int = ..., startcol: int = ..., freeze_panes: Optional[Any] = ...) -> None: ...

class _OpenpyxlReader(_BaseExcelReader):
    def __init__(self, filepath_or_buffer: FilePathOrBuffer[AnyStr]) -> None: ...
    def load_workbook(self, filepath_or_buffer: FilePathOrBuffer[AnyStr]) -> Any: ...
    def close(self) -> None: ...
    @property
    def sheet_names(self) -> List[str]: ...
    def get_sheet_by_name(self, name: str) -> Any: ...
    def get_sheet_by_index(self, index: int) -> Any: ...
    def get_sheet_data(self, sheet: Any, convert_float: bool) -> List[List[Scalar]]: ...



from datetime import datetime
from pandas._typing import ArrayLike as ArrayLike, TimestampConvertible
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCDatetimeIndex as ABCDatetimeIndex, ABCIndex as ABCIndex, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from typing import Any, Optional, TypeVar, Union, List, Tuple

ArrayConvertible = Union[List[Any], Tuple[Any], ArrayLike, ABCSeries]
Scalar = Union[int, float, str]
DatetimeScalar = TypeVar('DatetimeScalar', Scalar, datetime)
DatetimeScalarOrArrayConvertible = Union[DatetimeScalar, List[Any], Tuple[Any], ArrayLike, ABCSeries]

def should_cache(arg: ArrayConvertible, unique_share: float=..., check_count: Optional[int]=...) -> bool: ...
def to_datetime(arg: Any, errors: str = ..., dayfirst: bool = ..., yearfirst: bool = ..., utc: Optional[Any] = ..., format: Optional[Any] = ..., exact: bool = ..., unit: Optional[Any] = ..., infer_datetime_format: bool = ..., origin: TimestampConvertible = ..., cache: bool = ...) -> Any: ...
def to_time(arg: Any, format: Optional[Any] = ..., infer_time_format: bool = ..., errors: str = ...) -> Any: ...



import numpy as np
from pandas._typing import ArrayLike as ArrayLike
from pandas.core import ops as ops
from pandas.core.algorithms import unique as unique

from pandas.core.dtypes.dtypes import ExtensionDtype as ExtensionDtype
from pandas.core.dtypes.generic import ABCExtensionArray as ABCExtensionArray, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from pandas.core.missing import backfill_1d as backfill_1d, pad_1d as pad_1d
from pandas.core.sorting import nargsort as nargsort
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.util._decorators import Appender as Appender, Substitution as Substitution
from pandas.util._validators import validate_fillna_kwargs as validate_fillna_kwargs
from typing import Any, Optional, Sequence, Tuple, Union

def try_cast_to_ea(cls_or_instance: Any, obj: Any, dtype: Optional[Any] = ...) -> Any: ...

class ExtensionArray:
    def __getitem__(self, item: Any) -> None: ...
    def __setitem__(self, key: Union[int, np.ndarray], value: Any) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Any: ...
    def to_numpy(self, dtype: Optional[Any] = ..., copy: bool = ..., na_value: Any = ...) -> Any: ...
    @property
    def dtype(self) -> ExtensionDtype: ...
    @property
    def shape(self) -> Tuple[int, ...]: ...
    @property
    def ndim(self) -> int: ...
    @property
    def nbytes(self) -> int: ...
    def astype(self, dtype: Any, copy: bool = ...) -> Any: ...
    def isna(self) -> ArrayLike: ...
    def argsort(self, ascending: bool=..., kind: str=..., *args: Any, **kwargs: Any) -> np.ndarray: ...
    def fillna(self, value: Optional[Any] = ..., method: Optional[Any] = ..., limit: Optional[Any] = ...) -> Any: ...
    def dropna(self) -> Any: ...
    def shift(self, periods: int=..., fill_value: object=...) -> ABCExtensionArray: ...
    def unique(self) -> Any: ...
    def searchsorted(self, value: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
    def factorize(self, na_sentinel: int=...) -> Tuple[np.ndarray, ABCExtensionArray]: ...
    def repeat(self, repeats: Any, axis: Optional[Any] = ...) -> Any: ...
    def take(self, indices: Sequence[int], allow_fill: bool=..., fill_value: Any = ...) -> ABCExtensionArray: ...
    def copy(self) -> ABCExtensionArray: ...
    def view(self, dtype: Any = ...) -> Union[ABCExtensionArray, np.ndarray]: ...
    def ravel(self, order: Any = ...) -> ABCExtensionArray: ...

class ExtensionOpsMixin: ...
class ExtensionScalarOpsMixin(ExtensionOpsMixin): ...



import numpy as np
from pandas._libs.tslibs import Timestamp as Timestamp
from pandas.tseries.offsets import DateOffset as DateOffset, Tick as Tick, generate_range as generate_range
from typing import Tuple

def generate_regular_range(start: Timestamp, end: Timestamp, periods: int, freq: DateOffset) -> Tuple[np.ndarray, str]: ...



import numpy as np
from pandas import DataFrame as DataFrame, DatetimeIndex as DatetimeIndex, Index as Index, Int64Index as Int64Index, MultiIndex as MultiIndex, PeriodIndex as PeriodIndex, Series as Series, TimedeltaIndex as TimedeltaIndex, concat as concat, isna as isna
from pandas._config import config as config, get_option as get_option
from pandas._libs.tslibs import timezones as timezones
from pandas._typing import ArrayLike as ArrayLike, FrameOrSeries as FrameOrSeries
from pandas.core.arrays.categorical import Categorical as Categorical
from pandas.core.computation.pytables import PyTablesExpr as PyTablesExpr, maybe_expression as maybe_expression
from pandas.core.dtypes.common import ensure_object as ensure_object, is_categorical_dtype as is_categorical_dtype, is_complex_dtype as is_complex_dtype, is_datetime64_dtype as is_datetime64_dtype, is_datetime64tz_dtype as is_datetime64tz_dtype, is_extension_array_dtype as is_extension_array_dtype, is_list_like as is_list_like, is_string_dtype as is_string_dtype, is_timedelta64_dtype as is_timedelta64_dtype
from pandas.core.dtypes.generic import ABCExtensionArray as ABCExtensionArray
from pandas.core.dtypes.missing import array_equivalent as array_equivalent
from pandas.core.indexes.api import ensure_index as ensure_index
from pandas.errors import PerformanceWarning as PerformanceWarning
from pandas.io.common import stringify_path as stringify_path
from pandas.io.formats.printing import adjoin as adjoin, pprint_thing as pprint_thing
from pandas.util._decorators import cache_readonly as cache_readonly
# This from the PyTables package
from tables import Col as Col, File as File, Node as Node   # type: ignore
from typing import Any, Dict, Hashable, List, Optional, Tuple, Type, Union

Term = PyTablesExpr

class PossibleDataLossError(Exception): ...
class ClosedFileError(Exception): ...
class IncompatibilityWarning(Warning): ...

incompatibility_doc: str

class AttributeConflictWarning(Warning): ...

attribute_conflict_doc: str

class DuplicateWarning(Warning): ...

duplicate_doc: str
performance_doc: str
dropna_doc: str
format_doc: str

def to_hdf(path_or_buf: Any, key: str, value: FrameOrSeries, mode: str=..., complevel: Optional[int]=..., complib: Optional[str]=..., append: bool=..., format: Optional[str]=..., index: bool=..., min_itemsize: Optional[Union[int, Dict[str, int]]]=..., nan_rep: Any = ..., dropna: Optional[bool]=..., data_columns: Optional[List[str]]=..., errors: str=..., encoding: str=...) -> Any: ...
def read_hdf(path_or_buf: Any, key: Any = ..., mode: str=..., errors: str=..., where: Any = ..., start: Optional[int]=..., stop: Optional[int]=..., columns: Any = ..., iterator: Any = ..., chunksize: Optional[int]=..., **kwargs: Any) -> Any: ...

class HDFStore:
    def __init__(self, path: Any, mode: str=..., complevel: Optional[int]=..., complib: Any = ..., fletcher32: bool=..., **kwargs: Any) -> None: ...
    def __fspath__(self) -> Any: ...
    @property
    def root(self) -> Any: ...
    @property
    def filename(self) -> Any: ...
    def __getitem__(self, key: str) -> Any: ...
    def __setitem__(self, key: str, value: Any) -> Any: ...
    def __delitem__(self, key: str) -> Any: ...
    def __getattr__(self, name: str) -> Any: ...
    def __contains__(self, key: str) -> bool: ...
    def __len__(self) -> int: ...
    def __enter__(self) -> Any: ...
    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None: ...
    def keys(self) -> List[str]: ...
    def __iter__(self) -> Any: ...
    def items(self) -> None: ...
    iteritems: Any = ...
    def open(self, mode: str=..., **kwargs: Any) -> Any: ...
    def close(self) -> None: ...
    @property
    def is_open(self) -> bool: ...
    def flush(self, fsync: bool=...) -> Any: ...
    def get(self, key: str) -> Any: ...
    def select(self, key: str, where: Any = ..., start: Any = ..., stop: Any = ..., columns: Any = ..., iterator: Any = ..., chunksize: Any = ..., auto_close: bool=...) -> Any: ...
    def select_as_coordinates(self, key: str, where: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...
    def select_column(self, key: str, column: str, start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...
    def select_as_multiple(self, keys: Any, where: Any = ..., selector: Any = ..., columns: Any = ..., start: Any = ..., stop: Any = ..., iterator: Any = ..., chunksize: Any = ..., auto_close: bool=...) -> Any: ...
    def put(self, key: str, value: FrameOrSeries, format: Any = ..., index: Any = ..., append: Any = ..., complib: Any = ..., complevel: Optional[int]=..., min_itemsize: Optional[Union[int, Dict[str, int]]]=..., nan_rep: Any = ..., data_columns: Optional[List[str]]=..., encoding: Any = ..., errors: str=...) -> Any: ...
    def remove(self, key: str, where: Any = ..., start: Any = ..., stop: Any = ...) -> Any: ...
    def append(self, key: str, value: FrameOrSeries, format: Any = ..., axes: Any = ..., index: Any = ..., append: Any = ..., complib: Any = ..., complevel: Optional[int]=..., columns: Any = ..., min_itemsize: Optional[Union[int, Dict[str, int]]]=..., nan_rep: Any = ..., chunksize: Any = ..., expectedrows: Any = ..., dropna: Optional[bool]=..., data_columns: Optional[List[str]]=..., encoding: Any = ..., errors: str=...) -> Any: ...
    def append_to_multiple(self, d: Dict[Any, Any], value: Any, selector: Any, data_columns: Any = ..., axes: Any = ..., dropna: Any = ..., **kwargs: Any) -> Any: ...
    def create_table_index(self, key: str, columns: Any = ..., optlevel: Optional[int]=..., kind: Optional[str]=...) -> Any: ...
    def groups(self) -> Any: ...
    def walk(self, where: str = ...) -> None: ...
    def get_node(self, key: str) -> Optional[Node]: ...
    def get_storer(self, key: str) -> Union[GenericFixed, Table]: ...
    def copy(self, file: Any, mode: Any = ..., propindexes: bool=..., keys: Any = ..., complib: Any = ..., complevel: Optional[int]=..., fletcher32: bool=..., overwrite: Any = ...) -> Any: ...
    def info(self) -> str: ...

class TableIterator:
    chunksize: Optional[int]
    store: HDFStore
    s: Union[GenericFixed, Table]
    func: Any = ...
    where: Any = ...
    nrows: Any = ...
    start: Any = ...
    stop: Any = ...
    coordinates: Any = ...
    auto_close: Any = ...
    def __init__(self, store: HDFStore, s: Union[GenericFixed, Table], func: Any, where: Any, nrows: Any, start: Any = ..., stop: Any = ..., iterator: bool=..., chunksize: Optional[int]=..., auto_close: bool=...) -> None: ...
    def __iter__(self) -> Any: ...
    def close(self) -> None: ...
    def get_result(self, coordinates: bool=...) -> Any: ...

class IndexCol:
    is_an_indexable: bool = ...
    is_data_indexable: bool = ...
    name: str
    cname: str
    values: Any = ...
    kind: Any = ...
    typ: Any = ...
    axis: Any = ...
    pos: Any = ...
    freq: Any = ...
    tz: Any = ...
    index_name: Any = ...
    ordered: Any = ...
    table: Any = ...
    meta: Any = ...
    metadata: Any = ...
    def __init__(self, name: str, values: Any = ..., kind: Any = ..., typ: Any = ..., cname: Optional[str]=..., axis: Any = ..., pos: Any = ..., freq: Any = ..., tz: Any = ..., index_name: Any = ..., ordered: Any = ..., table: Any = ..., meta: Any = ..., metadata: Any = ...) -> None: ...
    @property
    def itemsize(self) -> int: ...
    @property
    def kind_attr(self) -> str: ...
    def set_pos(self, pos: int) -> Any: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    @property
    def is_indexed(self) -> bool: ...
    def convert(self, values: np.ndarray, nan_rep: Any, encoding: str, errors: str) -> Any: ...
    def take_data(self) -> Any: ...
    @property
    def attrs(self) -> Any: ...
    @property
    def description(self) -> Any: ...
    @property
    def col(self) -> Any: ...
    @property
    def cvalues(self) -> Any: ...
    def __iter__(self) -> Any: ...
    def maybe_set_size(self, min_itemsize: Optional[Any] = ...) -> None: ...
    def validate_names(self) -> None: ...
    def validate_and_set(self, handler: AppendableTable, append: bool) -> Any: ...
    def validate_col(self, itemsize: Optional[Any] = ...) -> Any: ...
    def validate_attr(self, append: bool) -> Any: ...
    def update_info(self, info: Any) -> None: ...
    def set_info(self, info: Any) -> None: ...
    def set_attr(self) -> None: ...
    def validate_metadata(self, handler: AppendableTable) -> Any: ...
    def write_metadata(self, handler: AppendableTable) -> Any: ...

class GenericIndexCol(IndexCol):
    @property
    def is_indexed(self) -> bool: ...
    def convert(self, values: np.ndarray, nan_rep: Any, encoding: str, errors: str) -> Any: ...
    def set_attr(self) -> None: ...

class DataCol(IndexCol):
    is_an_indexable: bool = ...
    is_data_indexable: bool = ...
    dtype: Any = ...
    data: Any = ...
    def __init__(self, name: str, values: Any = ..., kind: Any = ..., typ: Any = ..., cname: Any = ..., pos: Any = ..., tz: Any = ..., ordered: Any = ..., table: Any = ..., meta: Any = ..., metadata: Any = ..., dtype: Any = ..., data: Any = ...) -> None: ...
    @property
    def dtype_attr(self) -> str: ...
    @property
    def meta_attr(self) -> str: ...
    def __eq__(self, other: Any) -> bool: ...
    kind: Any = ...
    def set_data(self, data: Union[np.ndarray, ABCExtensionArray]) -> Any: ...
    def take_data(self) -> Any: ...
    @classmethod
    def get_atom_string(cls, shape: Any, itemsize: Any) -> Any: ...
    @classmethod
    def get_atom_coltype(cls: Any, kind: str) -> Type[Col]: ...
    @classmethod
    def get_atom_data(cls: Any, shape: Any, kind: str) -> Col: ...
    @classmethod
    def get_atom_datetime64(cls, shape: Any) -> Any: ...
    @classmethod
    def get_atom_timedelta64(cls, shape: Any) -> Any: ...
    @property
    def shape(self) -> Any: ...
    @property
    def cvalues(self) -> Any: ...
    def validate_attr(self, append: Any) -> None: ...
    def convert(self, values: np.ndarray, nan_rep: Any, encoding: str, errors: str) -> Any: ...
    def set_attr(self) -> None: ...

class DataIndexableCol(DataCol):
    is_data_indexable: bool = ...
    def validate_names(self) -> None: ...
    @classmethod
    def get_atom_string(cls, shape: Any, itemsize: Any) -> Any: ...
    @classmethod
    def get_atom_data(cls: Any, shape: Any, kind: str) -> Col: ...
    @classmethod
    def get_atom_datetime64(cls, shape: Any) -> Any: ...
    @classmethod
    def get_atom_timedelta64(cls, shape: Any) -> Any: ...

class GenericDataIndexableCol(DataIndexableCol): ...

class Fixed:
    pandas_kind: str
    format_type: str = ...
    obj_type: Type[Union[DataFrame, Series]]
    ndim: int
    encoding: str
    parent: HDFStore
    group: Node
    errors: str
    is_table: bool = ...
    def __init__(self, parent: HDFStore, group: Node, encoding: str=..., errors: str=...) -> None: ...
    @property
    def is_old_version(self) -> bool: ...
    @property
    def version(self) -> Tuple[int, int, int]: ...
    @property
    def pandas_type(self) -> Any: ...
    def set_object_info(self) -> None: ...
    def copy(self) -> Any: ...
    @property
    def shape(self) -> Any: ...
    @property
    def pathname(self) -> Any: ...
    @property
    def attrs(self) -> Any: ...
    def set_attrs(self) -> None: ...
    def get_attrs(self) -> None: ...
    @property
    def storable(self) -> Any: ...
    @property
    def is_exists(self) -> bool: ...
    @property
    def nrows(self) -> Any: ...
    def validate(self, other: Any) -> Any: ...
    def validate_version(self, where: Optional[Any] = ...) -> Any: ...
    def infer_axes(self) -> Any: ...
    def read(self, where: Any = ..., columns: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...
    def write(self, **kwargs: Any) -> None: ...
    def delete(self, where: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...

class GenericFixed(Fixed):
    attributes: List[str] = ...
    def validate_read(self, columns: Any, where: Any) -> None: ...
    @property
    def is_exists(self) -> bool: ...
    def set_attrs(self) -> None: ...
    encoding: Any = ...
    errors: Any = ...
    def get_attrs(self) -> None: ...
    def write(self, obj: Any, **kwargs: Any) -> None: ...   # type: ignore
    def read_array(self, key: str, start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...
    def read_index(self, key: str, start: Optional[int]=..., stop: Optional[int]=...) -> Index: ...
    def write_index(self, key: str, index: Index) -> Any: ...
    def write_multi_index(self, key: str, index: MultiIndex) -> Any: ...
    def read_multi_index(self, key: str, start: Optional[int]=..., stop: Optional[int]=...) -> MultiIndex: ...
    def read_index_node(self, node: Node, start: Optional[int]=..., stop: Optional[int]=...) -> Index: ...
    def write_array_empty(self, key: str, value: ArrayLike) -> Any: ...
    def write_array(self, key: str, value: ArrayLike, items: Optional[Index]=...) -> Any: ...

class SeriesFixed(GenericFixed):
    pandas_kind: str = ...
    attributes: Any = ...
    name: Optional[Hashable]
    @property
    def shape(self) -> Any: ...
    def read(self, where: Any = ..., columns: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...
    def write(self, obj: Any, **kwargs: Any) -> None: ...   # type: ignore

class BlockManagerFixed(GenericFixed):
    attributes: Any = ...
    nblocks: int
    @property
    def shape(self) -> Any: ...
    def read(self, where: Any = ..., columns: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...
    def write(self, obj: Any, **kwargs: Any) -> None: ...   # type: ignore

class FrameFixed(BlockManagerFixed):
    pandas_kind: str = ...
    obj_type: Any = ...

class Table(Fixed):
    pandas_kind: str = ...
    format_type: str = ...
    table_type: str
    levels: int = ...
    is_table: bool = ...
    index_axes: List[IndexCol]
    non_index_axes: List[Tuple[int, Any]]
    values_axes: List[DataCol]
    data_columns: List[Any]
    metadata: List[Any]
    info: Dict[Any, Any]
    nan_rep: Any = ...
    def __init__(self, parent: HDFStore, group: Node, encoding: Any = ..., errors: str=..., index_axes: Any = ..., non_index_axes: Any = ..., values_axes: Any = ..., data_columns: Any = ..., info: Any = ..., nan_rep: Any = ...) -> None: ...
    @property
    def table_type_short(self) -> str: ...
    def __getitem__(self, c: str) -> Any: ...
    def validate(self, other: Any) -> None: ...
    @property
    def is_multi_index(self) -> bool: ...
    def validate_multiindex(self, obj: Any) -> Any: ...
    @property
    def nrows_expected(self) -> int: ...
    @property
    def is_exists(self) -> bool: ...
    @property
    def storable(self) -> Any: ...
    @property
    def table(self) -> Any: ...
    @property
    def dtype(self) -> Any: ...
    @property
    def description(self) -> Any: ...
    @property
    def axes(self) -> Any: ...
    @property
    def ncols(self) -> int: ...
    @property
    def is_transposed(self) -> bool: ...
    @property
    def data_orientation(self) -> Any: ...
    def queryables(self) -> Dict[str, Any]: ...
    def index_cols(self) -> Any: ...
    def values_cols(self) -> List[str]: ...
    def write_metadata(self, key: str, values: np.ndarray) -> Any: ...
    def read_metadata(self, key: str) -> Any: ...
    def set_attrs(self) -> None: ...
    encoding: Any = ...
    errors: Any = ...
    def get_attrs(self) -> None: ...
    def validate_version(self, where: Optional[Any] = ...) -> None: ...
    def validate_min_itemsize(self, min_itemsize: Any) -> None: ...
    def indexables(self) -> Any: ...
    def create_index(self, columns: Any = ..., optlevel: Any = ..., kind: Optional[str]=...) -> Any: ...
    @classmethod
    def get_object(cls: Any, obj: Any, transposed: bool) -> Any: ...
    def validate_data_columns(self, data_columns: Any, min_itemsize: Any, non_index_axes: Any) -> Any: ...
    def process_axes(self, obj: Any, selection: Selection, columns: Any = ...) -> Any: ...
    def create_description(self, complib: Any, complevel: Optional[int], fletcher32: bool, expectedrows: Optional[int]) -> Dict[str, Any]: ...
    def read_coordinates(self, where: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...
    def read_column(self, column: str, where: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...

class WORMTable(Table):
    table_type: str = ...
    def read(self, where: Any = ..., columns: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...
    def write(self, **kwargs: Any) -> None: ...

class AppendableTable(Table):
    table_type: str = ...
    def write(self, obj: Any, axes: Optional[Any] = ..., append: bool = ..., complib: Optional[Any] = ..., complevel: Optional[Any] = ..., fletcher32: Optional[Any] = ..., min_itemsize: Optional[Any] = ..., chunksize: Optional[Any] = ..., expectedrows: Optional[Any] = ..., dropna: bool = ..., nan_rep: Optional[Any] = ..., data_columns: Optional[Any] = ...) -> None: ...   # type: ignore
    def write_data(self, chunksize: Optional[int], dropna: bool=...) -> Any: ...
    def write_data_chunk(self, rows: np.ndarray, indexes: List[np.ndarray], mask: Optional[np.ndarray], values: List[np.ndarray]) -> Any: ...
    def delete(self, where: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...

class AppendableFrameTable(AppendableTable):
    pandas_kind: str = ...
    table_type: str = ...
    ndim: int = ...
    obj_type: Type[Union[DataFrame, Series]] = ...
    @property
    def is_transposed(self) -> bool: ...
    @classmethod
    def get_object(cls: Any, obj: Any, transposed: bool) -> Any: ...
    def read(self, where: Any = ..., columns: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...

class AppendableSeriesTable(AppendableFrameTable):
    pandas_kind: str = ...
    table_type: str = ...
    ndim: int = ...
    obj_type: Any = ...
    @property
    def is_transposed(self) -> bool: ...
    @classmethod
    def get_object(cls: Any, obj: Any, transposed: bool) -> Any: ...
    def write(self, obj: Any, data_columns: Optional[Any] = ..., **kwargs: Any) -> Any: ...   # type: ignore
    def read(self, where: Any = ..., columns: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Series: ...

class AppendableMultiSeriesTable(AppendableSeriesTable):
    pandas_kind: str = ...
    table_type: str = ...
    def write(self, obj: Any, **kwargs: Any) -> Any: ...   # type: ignore

class GenericTable(AppendableFrameTable):
    pandas_kind: str = ...
    table_type: str = ...
    ndim: int = ...
    obj_type: Any = ...
    @property
    def pandas_type(self) -> str: ...
    @property
    def storable(self) -> Any: ...
    non_index_axes: Any = ...
    nan_rep: Any = ...
    levels: Any = ...
    index_axes: Any = ...
    values_axes: Any = ...
    data_columns: Any = ...
    def get_attrs(self) -> None: ...
    def indexables(self) -> Any: ...
    def write(self, **kwargs: Any) -> None: ...   # type: ignore

class AppendableMultiFrameTable(AppendableFrameTable):
    table_type: str = ...
    obj_type: Any = ...
    ndim: int = ...
    @property
    def table_type_short(self) -> str: ...
    def write(self, obj: Any, data_columns: Optional[Any] = ..., **kwargs: Any) -> Any: ...   # type: ignore
    def read(self, where: Any = ..., columns: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> Any: ...

class Selection:
    table: Any = ...
    where: Any = ...
    start: Any = ...
    stop: Any = ...
    condition: Any = ...
    filter: Any = ...
    terms: Any = ...
    coordinates: Any = ...
    def __init__(self, table: Table, where: Any = ..., start: Optional[int]=..., stop: Optional[int]=...) -> None: ...
    def generate(self, where: Any) -> Any: ...
    def select(self) -> Any: ...
    def select_coords(self) -> Any: ...



from pandas.core.arrays.base import ExtensionArray as ExtensionArray, ExtensionOpsMixin as ExtensionOpsMixin, ExtensionScalarOpsMixin as ExtensionScalarOpsMixin, try_cast_to_ea as try_cast_to_ea
from pandas.core.arrays.boolean import BooleanArray as BooleanArray
from pandas.core.arrays.categorical import Categorical as Categorical
from pandas.core.arrays.datetimes import DatetimeArray as DatetimeArray
from pandas.core.arrays.integer import IntegerArray as IntegerArray, integer_array as integer_array
from pandas.core.arrays.interval import IntervalArray as IntervalArray
from pandas.core.arrays.numpy_ import PandasArray as PandasArray, PandasDtype as PandasDtype
from pandas.core.arrays.period import PeriodArray as PeriodArray, period_array as period_array
from pandas.core.arrays.sparse import SparseArray as SparseArray
from pandas.core.arrays.string_ import StringArray as StringArray
from pandas.core.arrays.timedeltas import TimedeltaArray as TimedeltaArray



from __future__ import annotations

import sys

from pandas._typing import AggregationFunction, GeneralDuplicatesKeepStrategy
from pandas.core import algorithms as algorithms
from pandas.core.accessor import DirNamesMixin as DirNamesMixin
from pandas.core.algorithms import duplicated as duplicated, unique1d as unique1d, value_counts as value_counts
from pandas.core.arrays import ExtensionArray as ExtensionArray
from pandas.core.construction import create_series_with_explicit_dtype as create_series_with_explicit_dtype
from pandas.core.dtypes.cast import is_nested_object as is_nested_object

from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCIndexClass as ABCIndexClass, ABCSeries as ABCSeries
from pandas.core.dtypes.missing import isna as isna
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.util._decorators import Appender as Appender, Substitution as Substitution, cache_readonly as cache_readonly
from pandas.util._validators import validate_bool_kwarg as validate_bool_kwarg
from typing import Any, Optional, overload

if sys.version_info >= (3, 8):
    from typing import Literal
else:
    from typing_extensions import Literal

class PandasObject(DirNamesMixin):
    def __sizeof__(self) -> Any: ...

class NoNewAttributesMixin:
    def __setattr__(self, key: Any, value: Any) -> None: ...

class GroupByError(Exception): ...
class DataError(GroupByError): ...
class SpecificationError(GroupByError): ...

class SelectionMixin:
    def ndim(self) -> int: ...
    def __getitem__(self, key: Any) -> Any: ...
    def aggregate(self, func: AggregationFunction, *args: Any, **kwargs: Any) -> Any: ...
    def agg(self, func: AggregationFunction, *args: Any, **kwargs: Any) -> Any: ...

class ShallowMixin: ...

class IndexOpsMixin:
    T: Any = ...
    to_list: Any = ...
    is_monotonic_increasing: Any = ...
    __array_priority__: int = ...
    def __iter__(self) -> Any: ...
    def transpose(self, *args: Any, **kwargs: Any) -> Any: ...
    @property
    def shape(self) -> Any: ...
    @property
    def ndim(self) -> int: ...
    def item(self) -> Any: ...
    @property
    def nbytes(self) -> Any: ...
    @property
    def size(self) -> Any: ...
    @property
    def array(self) -> ExtensionArray: ...
    def to_numpy(self, dtype: Optional[Any] = ..., copy: bool = ..., na_value: Any = ..., **kwargs: Any) -> Any: ...
    @property
    def empty(self) -> Any: ...
    def max(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def argmax(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def min(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def argmin(self, axis: Optional[Any] = ..., skipna: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def tolist(self) -> Any: ...
    def hasnans(self) -> Any: ...
    def value_counts(self, normalize: bool = ..., sort: bool = ..., ascending: bool = ..., bins: Optional[Any] = ..., dropna: bool = ...) -> Any: ...
    def unique(self) -> Any: ...
    def nunique(self, dropna: bool = ...) -> Any: ...
    @property
    def is_unique(self) -> Any: ...
    @property
    def is_monotonic(self) -> Any: ...
    @property
    def is_monotonic_decreasing(self) -> bool: ...
    def memory_usage(self, deep: bool = ...) -> Any: ...
    def factorize(self, sort: bool = ..., na_sentinel: int = ...) -> Any: ...
    def searchsorted(self, value: Any, side: str = ..., sorter: Optional[Any] = ...) -> Any: ...
    @overload
    def drop_duplicates(self, keep: GeneralDuplicatesKeepStrategy = ..., *, inplace: Literal[False] = ...) -> Any: ...
    @overload
    def drop_duplicates(self, keep: GeneralDuplicatesKeepStrategy = ..., *, inplace: Literal[True]) -> None: ...
    def duplicated(self, keep: GeneralDuplicatesKeepStrategy = ...) -> Any: ...



import abc
from pandas.core.computation.align import align_terms as align_terms, reconstruct_object as reconstruct_object
from typing import Any

class NumExprClobberingError(NameError): ...

class AbstractEngine(metaclass=abc.ABCMeta):
    has_neg_frac: bool = ...
    expr: Any = ...
    aligned_axes: Any = ...
    result_type: Any = ...
    def __init__(self, expr: Any) -> None: ...
    def convert(self) -> str: ...
    def evaluate(self) -> object: ...

class NumExprEngine(AbstractEngine):
    has_neg_frac: bool = ...

class PythonEngine(AbstractEngine):
    has_neg_frac: bool = ...
    def evaluate(self) -> Any: ...



import numpy as np
from pandas._libs.indexing import _NDFrameIndexerBase

from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame, ABCMultiIndex as ABCMultiIndex, ABCSeries as ABCSeries
from pandas.core.indexes.api import Index as Index, InvalidIndexError as InvalidIndexError
from pandas.errors import AbstractMethodError as AbstractMethodError
from pandas.util._decorators import Appender as Appender
from typing import Any, Optional

class _IndexSlice:
    def __getitem__(self, arg: Any) -> Any: ...

IndexSlice: Any

class IndexingError(Exception): ...

class IndexingMixin:
    @property
    def iloc(self) -> _iLocIndexer: ...
    @property
    def loc(self) -> _LocIndexer: ...
    @property
    def at(self) -> _AtIndexer: ...
    @property
    def iat(self) -> _iAtIndexer: ...

class _NDFrameIndexer(_NDFrameIndexerBase):
    axis: Any = ...
    def __call__(self, axis: Optional[Any] = ...) -> Any: ...
    def __getitem__(self, key: Any) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...

class _LocationIndexer(_NDFrameIndexer):
    def __getitem__(self, key: Any) -> Any: ...

class _LocIndexer(_LocationIndexer): ...
class _iLocIndexer(_LocationIndexer): ...

class _ScalarAccessIndexer(_NDFrameIndexerBase):
    def __getitem__(self, key: Any) -> Any: ...
    def __setitem__(self, key: Any, value: Any) -> None: ...

class _AtIndexer(_ScalarAccessIndexer): ...
class _iAtIndexer(_ScalarAccessIndexer): ...

def convert_to_index_sliceable(obj: Any, key: Any) -> Any: ...
def check_bool_indexer(index: Index, key: Any) -> np.ndarray: ...
def convert_missing_indexer(indexer: Any) -> Any: ...
def convert_from_missing_indexer_tuple(indexer: Any, axes: Any) -> Any: ...
def maybe_convert_ix(*args: Any) -> Any: ...
def is_nested_tuple(tup: Any, labels: Any) -> bool: ...
def is_label_like(key: Any) -> bool: ...
def need_slice(obj: Any) -> bool: ...



from pandas._config import describe_option as describe_option, get_option as get_option, option_context as option_context, options as options, reset_option as reset_option, set_option as set_option
from pandas.core.api import BooleanDtype as BooleanDtype, Categorical as Categorical, CategoricalDtype as CategoricalDtype, CategoricalIndex as CategoricalIndex, DataFrame as DataFrame, DateOffset as DateOffset, DatetimeIndex as DatetimeIndex, DatetimeTZDtype as DatetimeTZDtype, Float64Index as Float64Index, Grouper as Grouper, Index as Index, IndexSlice as IndexSlice, Int16Dtype as Int16Dtype, Int32Dtype as Int32Dtype, Int64Dtype as Int64Dtype, Int64Index as Int64Index, Int8Dtype as Int8Dtype, Interval as Interval, IntervalDtype as IntervalDtype, IntervalIndex as IntervalIndex, MultiIndex as MultiIndex, NA as NA,  NamedAgg as NamedAgg, PeriodDtype as PeriodDtype, PeriodIndex as PeriodIndex, RangeIndex as RangeIndex, Series as Series, StringDtype as StringDtype, Timedelta as Timedelta, TimedeltaIndex as TimedeltaIndex, Timestamp as Timestamp, UInt16Dtype as UInt16Dtype, UInt32Dtype as UInt32Dtype, UInt64Dtype as UInt64Dtype, UInt64Index as UInt64Index, UInt8Dtype as UInt8Dtype, array as array, bdate_range as bdate_range, date_range as date_range, factorize as factorize, interval_range as interval_range, isna as isna, isnull as isnull, notna as notna, notnull as notnull, period_range as period_range, set_eng_float_format as set_eng_float_format, timedelta_range as timedelta_range, to_datetime as to_datetime, to_numeric as to_numeric, to_timedelta as to_timedelta, unique as unique, value_counts as value_counts
from pandas.core.arrays.sparse import SparseDtype as SparseDtype
from pandas.core.computation.api import eval as eval
from pandas.core.reshape.api import concat as concat, crosstab as crosstab, cut as cut, get_dummies as get_dummies, lreshape as lreshape, melt as melt, merge as merge, merge_asof as merge_asof, merge_ordered as merge_ordered, pivot as pivot, pivot_table as pivot_table, qcut as qcut, wide_to_long as wide_to_long
from pandas.io.api import ExcelFile as ExcelFile, ExcelWriter as ExcelWriter, HDFStore as HDFStore, read_clipboard as read_clipboard, read_csv as read_csv, read_excel as read_excel, read_feather as read_feather, read_fwf as read_fwf, read_gbq as read_gbq, read_hdf as read_hdf, read_html as read_html, read_json as read_json, read_orc as read_orc, read_parquet as read_parquet, read_pickle as read_pickle, read_sas as read_sas, read_spss as read_spss, read_sql as read_sql, read_sql_query as read_sql_query, read_sql_table as read_sql_table, read_stata as read_stata, read_table as read_table, to_pickle as to_pickle
from pandas.tseries import offsets as offsets
from pandas.tseries.api import infer_freq as infer_freq
from pandas.util._print_versions import show_versions as show_versions
from pandas.util._tester import test as test
from typing import Any

__docformat__: str
hard_dependencies: Any
missing_dependencies: Any
module: Any
v: Any
__git_version__: Any

def __getattr__(name: Any) -> Any: ...

class Panel: ...
class SparseDataFrame: ...
class SparseSeries: ...

class __numpy:
    np: Any = ...
    warnings: Any = ...
    def __init__(self) -> None: ...
    def __getattr__(self, item: Any) -> Any: ...

np: Any

class __Datetime(type):
    datetime: Any = ...
    def __getattr__(cls, item: Any) -> Any: ...
    def __instancecheck__(cls, other: Any) -> Any: ...

class __DatetimeSub(metaclass=__Datetime):
    def emit_warning(self, dummy: int = ...) -> None: ...
    def __new__(cls, *args: Any, **kwargs: Any) -> Any: ...

datetime: Any

class __SparseArray(type):
    SparseArray: Any = ...
    def __instancecheck__(cls, other: Any) -> Any: ...

class __SparseArraySub(metaclass=__SparseArray):
    def emit_warning(self, dummy: int = ...) -> None: ...
    def __new__(cls, *args: Any, **kwargs: Any) -> Any: ...

SparseArray: Any



import datetime
from collections import abc
from pandas import Categorical as Categorical, DatetimeIndex as DatetimeIndex,  Timestamp as Timestamp, concat as concat, isna as isna, to_datetime as to_datetime, to_timedelta as to_timedelta
from pandas._typing import FilePathOrBuffer as FilePathOrBuffer
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.series import Series as Series
from pandas.util._decorators import Appender as Appender
from typing import Any, Dict, Hashable, Optional, Sequence, AnyStr


def read_stata(filepath_or_buffer: Any, convert_dates: bool = ..., convert_categoricals: bool = ..., index_col: Optional[Any] = ..., convert_missing: bool = ..., preserve_dtypes: bool = ..., columns: Optional[Any] = ..., order_categoricals: bool = ..., chunksize: Optional[Any] = ..., iterator: bool = ...) -> Any: ...

stata_epoch: Any
excessive_string_length_error: str

class PossiblePrecisionLoss(Warning): ...

precision_loss_doc: str

class ValueLabelTypeMismatch(Warning): ...

value_label_mismatch_doc: str

class InvalidColumnName(Warning): ...

invalid_name_doc: str

class StataValueLabel:
    labname: Any = ...
    value_labels: Any = ...
    text_len: Any = ...
    off: Any = ...
    val: Any = ...
    txt: Any = ...
    n: int = ...
    len: Any = ...
    def __init__(self, catarray: Any, encoding: str = ...) -> None: ...
    def generate_value_label(self, byteorder: Any) -> Any: ...

class StataMissingValue:
    MISSING_VALUES: Any = ...
    bases: Any = ...
    float32_base: bytes = ...
    increment: Any = ...
    value: Any = ...
    int_value: Any = ...
    float64_base: bytes = ...
    BASE_MISSING_VALUES: Any = ...
    def __init__(self, value: Any) -> None: ...
    string: Any = ...
    def __eq__(self, other: Any) -> bool: ...
    @classmethod
    def get_base_missing_value(cls, dtype: Any) -> Any: ...

class StataParser:
    DTYPE_MAP: Any = ...
    DTYPE_MAP_XML: Any = ...
    TYPE_MAP: Any = ...
    TYPE_MAP_XML: Any = ...
    VALID_RANGE: Any = ...
    OLD_TYPE_MAPPING: Any = ...
    MISSING_VALUES: Any = ...
    NUMPY_TYPE_MAP: Any = ...
    RESERVED_WORDS: Any = ...
    def __init__(self) -> None: ...

class StataReader(StataParser, abc.Iterator[Any]):
    __doc__: Any = ...
    col_sizes: Any = ...
    path_or_buf: Any = ...
    def __init__(self, path_or_buf: Any, convert_dates: bool = ..., convert_categoricals: bool = ..., index_col: Optional[Any] = ..., convert_missing: bool = ..., preserve_dtypes: bool = ..., columns: Optional[Any] = ..., order_categoricals: bool = ..., chunksize: Optional[Any] = ...) -> None: ...
    def __enter__(self) -> Any: ...
    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None: ...
    def close(self) -> None: ...
    def __next__(self) -> Any: ...
    def get_chunk(self, size: Optional[Any] = ...) -> Any: ...
    def read(self, nrows: Optional[Any] = ..., convert_dates: Optional[Any] = ..., convert_categoricals: Optional[Any] = ..., index_col: Optional[Any] = ..., convert_missing: Optional[Any] = ..., preserve_dtypes: Optional[Any] = ..., columns: Optional[Any] = ..., order_categoricals: Optional[Any] = ...) -> Any: ...
    @property
    def data_label(self) -> Any: ...
    def variable_labels(self) -> Any: ...
    def value_labels(self) -> Any: ...

class StataWriter(StataParser):
    type_converters: Any = ...
    def __init__(self, fname: Any, data: Any, convert_dates: Optional[Any] = ..., write_index: bool = ..., byteorder: Optional[Any] = ..., time_stamp: Optional[Any] = ..., data_label: Optional[Any] = ..., variable_labels: Optional[Any] = ...) -> None: ...
    def write_file(self) -> None: ...

class StataStrLWriter:
    df: Any = ...
    columns: Any = ...
    def __init__(self, df: Any, columns: Any, version: int = ..., byteorder: Optional[Any] = ...) -> None: ...
    def generate_table(self) -> Any: ...
    def generate_blob(self, gso_table: Any) -> Any: ...

class StataWriter117(StataWriter):
    def __init__(self, fname: Any, data: Any, convert_dates: Optional[Any] = ..., write_index: bool = ..., byteorder: Optional[Any] = ..., time_stamp: Optional[Any] = ..., data_label: Optional[Any] = ..., variable_labels: Optional[Any] = ..., convert_strl: Optional[Any] = ...) -> None: ...

class StataWriterUTF8(StataWriter117):
    def __init__(self, fname: FilePathOrBuffer[AnyStr], data: DataFrame, convert_dates: Optional[Dict[Hashable, str]]=..., write_index: bool=..., byteorder: Optional[str]=..., time_stamp: Optional[datetime.datetime]=..., data_label: Optional[str]=..., variable_labels: Optional[Dict[Hashable, str]]=..., convert_strl: Optional[Sequence[Hashable]]=..., version: Optional[int]=...) -> None: ...



from pandas._typing import FilePathOrBuffer as FilePathOrBuffer, Scalar as Scalar
from pandas.io.excel._base import _BaseExcelReader
from typing import Any, List, AnyStr


class _ODFReader(_BaseExcelReader):
    def __init__(self, filepath_or_buffer: FilePathOrBuffer[AnyStr]) -> None: ...
    def load_workbook(self, filepath_or_buffer: FilePathOrBuffer[AnyStr]) -> Any: ...
    @property
    def empty_value(self) -> str: ...
    @property
    def sheet_names(self) -> List[str]: ...
    def get_sheet_by_index(self, index: int) -> Any: ...
    def get_sheet_by_name(self, name: str) -> Any: ...
    def get_sheet_data(self, sheet: Any, convert_float: bool) -> List[List[Scalar]]: ...



from pandas.core.groupby.generic import DataFrameGroupBy as DataFrameGroupBy, NamedAgg as NamedAgg, SeriesGroupBy as SeriesGroupBy
from pandas.core.groupby.groupby import GroupBy as GroupBy
from pandas.core.groupby.grouper import Grouper as Grouper



from pandas._testing import assert_extension_array_equal as assert_extension_array_equal, assert_frame_equal as assert_frame_equal, assert_index_equal as assert_index_equal, assert_series_equal as assert_series_equal



from pandas.core.accessor import PandasDelegate as PandasDelegate, delegate_names as delegate_names
from pandas.core.arrays.sparse.array import SparseArray as SparseArray
from pandas.core.arrays.sparse.dtype import SparseDtype as SparseDtype
from pandas.core.dtypes.cast import find_common_type as find_common_type
from typing import Any, Optional

class BaseAccessor:
    def __init__(self, data: Optional[Any] = ...) -> None: ...

class SparseAccessor(BaseAccessor, PandasDelegate):
    @classmethod
    def from_coo(cls, A: Any, dense_index: bool = ...) -> Any: ...
    def to_coo(self, row_levels: Any = ..., column_levels: Any = ..., sort_labels: bool = ...) -> Any: ...
    def to_dense(self) -> Any: ...

class SparseFrameAccessor(BaseAccessor, PandasDelegate):
    @classmethod
    def from_spmatrix(cls, data: Any, index: Optional[Any] = ..., columns: Optional[Any] = ...) -> Any: ...
    def to_dense(self) -> Any: ...
    def to_coo(self) -> Any: ...
    @property
    def density(self) -> float: ...



from typing import Any, Optional

bn: Any

def set_use_bottleneck(v: bool = ...) -> None: ...

class disallow:
    dtypes: Any = ...
    def __init__(self, *dtypes: Any) -> None: ...
    def check(self, obj: Any) -> bool: ...
    def __call__(self, f: Any) -> Any: ...

class bottleneck_switch:
    name: Any = ...
    kwargs: Any = ...
    def __init__(self, name: Optional[Any] = ..., **kwargs: Any) -> None: ...
    def __call__(self, alt: Any) -> Any: ...

def nanany(values: Any, axis: Any = ..., skipna: bool=..., mask: Any = ...) -> Any: ...
def nanall(values: Any, axis: Any = ..., skipna: bool=..., mask: Any = ...) -> Any: ...
def nansum(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., min_count: int = ..., mask: Optional[Any] = ...) -> Any: ...
def nanmean(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., mask: Optional[Any] = ...) -> Any: ...
def nanmedian(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., mask: Optional[Any] = ...) -> Any: ...
def nanstd(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., ddof: int = ..., mask: Optional[Any] = ...) -> Any: ...
def nanvar(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., ddof: int = ..., mask: Optional[Any] = ...) -> Any: ...
def nansem(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., ddof: int = ..., mask: Optional[Any] = ...) -> Any: ...

nanmin: Any
nanmax: Any

def nanargmax(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., mask: Optional[Any] = ...) -> Any: ...
def nanargmin(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., mask: Optional[Any] = ...) -> Any: ...
def nanskew(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., mask: Optional[Any] = ...) -> Any: ...
def nankurt(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., mask: Optional[Any] = ...) -> Any: ...
def nanprod(values: Any, axis: Optional[Any] = ..., skipna: bool = ..., min_count: int = ..., mask: Optional[Any] = ...) -> Any: ...
def nancorr(a: Any, b: Any, method: str = ..., min_periods: Optional[Any] = ...) -> Any: ...
def get_corr_func(method: Any) -> Any: ...
def nancov(a: Any, b: Any, min_periods: Optional[Any] = ...) -> Any: ...
def make_nancomp(op: Any) -> Any: ...

nangt: Any
nange: Any
nanlt: Any
nanle: Any
naneq: Any
nanne: Any

def nanpercentile(values: Any, q: Any, axis: Any, na_value: Any, mask: Any, ndim: Any, interpolation: Any) -> Any: ...



from pandas.core.base import DataError as DataError
from pandas.core.dtypes.generic import ABCDataFrame as ABCDataFrame
from pandas.core.window.common import zsqrt as zsqrt
from pandas.core.window.rolling import _Rolling
from pandas.util._decorators import Appender as Appender, Substitution as Substitution
from typing import Any, Optional

class EWM(_Rolling):
    obj: Any = ...
    com: Any = ...
    min_periods: Any = ...
    adjust: Any = ...
    ignore_na: Any = ...
    axis: Any = ...
    on: Any = ...
    def __init__(self, obj: Any, com: Optional[Any] = ..., span: Optional[Any] = ..., halflife: Optional[Any] = ..., alpha: Optional[Any] = ..., min_periods: int = ..., adjust: bool = ..., ignore_na: bool = ..., axis: int = ...) -> None: ...
    def aggregate(self, func: Any, *args: Any, **kwargs: Any) -> Any: ...
    agg: Any = ...
    def mean(self, *args: Any, **kwargs: Any) -> Any: ...
    def std(self, bias: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    vol: Any = ...
    def var(self, bias: bool = ..., *args: Any, **kwargs: Any) -> Any: ...
    def cov(self, other: Optional[Any] = ..., pairwise: Optional[Any] = ..., bias: bool = ..., **kwargs: Any) -> Any: ...
    def corr(self, other: Optional[Any] = ..., pairwise: Optional[Any] = ..., **kwargs: Any) -> Any: ...



from pandas._libs import Timedelta as Timedelta, Timestamp as Timestamp
from pandas._libs.missing import NA as NA
from pandas.core.algorithms import factorize as factorize, unique as unique, value_counts as value_counts
from pandas.core.arrays import Categorical as Categorical
from pandas.core.arrays.boolean import BooleanDtype as BooleanDtype
from pandas.core.arrays.integer import Int16Dtype as Int16Dtype, Int32Dtype as Int32Dtype, Int64Dtype as Int64Dtype, Int8Dtype as Int8Dtype, UInt16Dtype as UInt16Dtype, UInt32Dtype as UInt32Dtype, UInt64Dtype as UInt64Dtype, UInt8Dtype as UInt8Dtype
from pandas.core.arrays.string_ import StringDtype as StringDtype
from pandas.core.construction import array as array
from pandas.core.dtypes.dtypes import CategoricalDtype as CategoricalDtype, DatetimeTZDtype as DatetimeTZDtype, IntervalDtype as IntervalDtype, PeriodDtype as PeriodDtype
from pandas.core.dtypes.missing import isna as isna, isnull as isnull, notna as notna, notnull as notnull
from pandas.core.frame import DataFrame as DataFrame
from pandas.core.groupby import Grouper as Grouper, NamedAgg as NamedAgg
from pandas.core.indexes.api import CategoricalIndex as CategoricalIndex, DatetimeIndex as DatetimeIndex, Float64Index as Float64Index, Index as Index, Int64Index as Int64Index, IntervalIndex as IntervalIndex, MultiIndex as MultiIndex, PeriodIndex as PeriodIndex, RangeIndex as RangeIndex, TimedeltaIndex as TimedeltaIndex, UInt64Index as UInt64Index
from pandas.core.indexes.datetimes import bdate_range as bdate_range, date_range as date_range
from pandas.core.indexes.interval import Interval as Interval, interval_range as interval_range
from pandas.core.indexes.period import period_range as period_range
from pandas.core.indexes.timedeltas import timedelta_range as timedelta_range
from pandas.core.indexing import IndexSlice as IndexSlice
from pandas.core.series import Series as Series
from pandas.core.tools.datetimes import to_datetime as to_datetime
from pandas.core.tools.numeric import to_numeric as to_numeric
from pandas.core.tools.timedeltas import to_timedelta as to_timedelta
from pandas.io.formats.format import set_eng_float_format as set_eng_float_format
from pandas.tseries.offsets import DateOffset as DateOffset



from typing import Any, Optional

class CSSWarning(UserWarning): ...

class CSSResolver:
    def __call__(self, declarations_str: Any, inherited: Optional[Any] = ...) -> Any: ...
    UNIT_RATIOS: Any = ...
    FONT_SIZE_RATIOS: Any = ...
    MARGIN_RATIOS: Any = ...
    BORDER_WIDTH_RATIOS: Any = ...
    def size_to_pt(self, in_val: Any, em_pt: Optional[Any] = ..., conversions: Any = ...) -> Any: ...
    def atomize(self, declarations: Any) -> None: ...
    SIDE_SHORTHANDS: Any = ...
    SIDES: Any = ...
    expand_border_color: Any = ...
    expand_border_style: Any = ...
    expand_border_width: Any = ...
    expand_margin: Any = ...
    expand_padding: Any = ...
    def parse(self, declarations_str: str) -> Any: ...
